@@ -3,11 +3,11 @@
 *  C4 dynamic programming library - code for heuristics          *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2006.  All Rights Reserved.                *
+*  Copyright (C) 2000-2008.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
-*  GNU Lesser General Public License. See the file COPYING       *
-*  or http://www.fsf.org/copyleft/lesser.html for details        *
+*  GNU General Public License, version 3. See the file COPYING   *
+*  or http://www.gnu.org/licenses/gpl.txt for details            *
 *                                                                *
 *  If you use this code, please keep this notice intact.         *
 *                                                                *
@@ -20,6 +20,7 @@
 
 /**/
 
+#if 0
 gchar *Heuristic_Refinement_to_string(Heuristic_Refinement refinement){
     register gchar *name = NULL;
     switch(refinement){
@@ -66,6 +67,7 @@ static gchar *Heuristic_Argument_parse_refinement(gchar *arg_string,
     (*refinement) = Heuristic_Refinement_from_string(refine_str);
     return NULL;
     }
+#endif /* 0 */
 
 Heuristic_ArgumentSet *Heuristic_ArgumentSet_create(Argument *arg){
     register ArgumentSet *as;
@@ -94,12 +96,14 @@ Heuristic_ArgumentSet *Heuristic_ArgumentSet_create(Argument *arg){
             "External span range", "12",
             Argument_parse_int, &has.span_range_external);
         /**/
+#if 0
         ArgumentSet_add_option(as, '\0', "refine", NULL,
             "Alignment refinement strategy [none|full|region]", "none",
             Heuristic_Argument_parse_refinement, &has.refinement);
         ArgumentSet_add_option(as, '\0', "refineboundary", NULL,
             "Refinement region boundary", "32",
             Argument_parse_int, &has.refinement_boundary);
+#endif /* 0 */
         /**/
         Argument_absorb_ArgumentSet(arg, as);
         }
@@ -776,6 +780,7 @@ Heuristic *Heuristic_create(C4_Model *model){
     heuristic->ref_count = 1;
     heuristic->model = C4_Model_share(model);
     heuristic->has = Heuristic_ArgumentSet_create(NULL);
+#if 0
     if(heuristic->has->refinement != Heuristic_Refinement_NONE){
         heuristic->optimal = Optimal_create(model, NULL,
                              Optimal_Type_SCORE
@@ -784,6 +789,7 @@ Heuristic *Heuristic_create(C4_Model *model){
     } else {
         heuristic->optimal = NULL;
         }
+#endif /* 0 */
     /**/
     heuristic->match_total = 0;
     for(i = 0; i < model->portal_list->len; i++){
@@ -833,8 +839,10 @@ void Heuristic_destroy(Heuristic *heuristic){
             if(heuristic->pair_matrix[i][j])
                 Heuristic_Pair_destroy(
                           heuristic->pair_matrix[i][j]);
+#if 0
     if(heuristic->optimal)
         Optimal_destroy(heuristic->optimal);
+#endif /* 0 */
     C4_Model_destroy(heuristic->model);
     g_free(heuristic->match_list);
     g_free(heuristic->pair_matrix);
@@ -860,8 +868,11 @@ GPtrArray *Heuristic_get_Optimal_list(Heuristic *heuristic){
     register Heuristic_Span *heuristic_span;
     register GPtrArray *optimal_list = g_ptr_array_new();
     g_assert(heuristic);
+#if 0
     /* Refinement optimal is not included here,
-       as will be included for exhaustive alignment anyway */
+     * as will be included for exhaustive alignment anyway
+     */
+#endif /* 0 */
     for(i = 0; i < heuristic->match_total; i++){
         match = heuristic->match_list[i];
         heuristic_terminal = match->start_terminal;
