@@ -3,11 +3,11 @@
 *  C4 dynamic programming library - code for models              *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2006.  All Rights Reserved.                *
+*  Copyright (C) 2000-2008.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
-*  GNU Lesser General Public License. See the file COPYING       *
-*  or http://www.fsf.org/copyleft/lesser.html for details        *
+*  GNU General Public License, version 3. See the file COPYING   *
+*  or http://www.gnu.org/licenses/gpl.txt for details            *
 *                                                                *
 *  If you use this code, please keep this notice intact.         *
 *                                                                *
@@ -1964,6 +1964,14 @@ gboolean C4_Model_is_global(C4_Model *model){
     return TRUE;
     }
 
+gboolean C4_Model_is_local(C4_Model *model){
+    if(model->start_state->scope != C4_Scope_ANYWHERE)
+        return FALSE;
+    if(model->end_state->scope != C4_Scope_ANYWHERE)
+        return FALSE;
+    return TRUE;
+    }
+
 void C4_Model_remove_all_shadows(C4_Model *model){
     register C4_Shadow *shadow;
     g_assert(model->is_open);
