@@ -117,8 +117,10 @@ Codegen *Codegen_create(gchar *directory, gchar *name){
     c->object_path = g_strconcat(code_dir, G_DIR_SEPARATOR_S,
                                c->name, ".o", NULL);
     c->fp = fopen(c->code_path, "w");
-    if(!c->fp)
+    if(!c->fp){
+        perror("Writing codegen file");
         g_error("Could not write codegen code to [%s]", c->code_path);
+        }
     c->indent = 0;
     g_free(code_dir);
     return c;
@@ -206,10 +208,10 @@ void Codegen_compile(Codegen *c,
         g_warning("Reusing codegen object [%s]", c->object_path);
         return;
         }
-    tmp = (gchar*)g_getenv("C4_CC");
+    tmp = (gchar*)g_getenv("CC");
     if(tmp)
         cc_command = tmp;
-    tmp = (gchar*)g_getenv("C4_CFLAGS");
+    tmp = (gchar*)g_getenv("CFLAGS");
     if(tmp)
         cc_flags = tmp;
     /* FIXME: these enviroment variables should be merged
