@@ -75,10 +75,10 @@ HSPset_ArgumentSet *HSPset_ArgumentSet_create(Argument *arg){
             Argument_parse_int, &has.codon_wordlimit);
         /**/
         ArgumentSet_add_option(as, '\0', "geneseed", "threshold",
-             "Geneseed Threshold", "0", 
+             "Geneseed Threshold", "0",
              Argument_parse_int, &has.geneseed_threshold);
         ArgumentSet_add_option(as, '\0', "geneseedrepeat", "number",
-             "Seeds per diagonal required for geneseed HSP seeding", "2",
+             "Seeds per diagonal required for geneseed HSP seeding", "3",
              Argument_parse_int, &has.geneseed_repeat);
         /**/
         Argument_absorb_ArgumentSet(arg, as);
@@ -115,6 +115,8 @@ void HSP_Param_set_wordlen(HSP_Param *hsp_param, gint wordlen){
     return;
     }
 
+/**/
+
 void HSP_Param_set_dna_hsp_threshold(HSP_Param *hsp_param,
                                      gint dna_hsp_threshold){
     if(hsp_param->match->type == Match_Type_DNA2DNA)
@@ -138,6 +140,33 @@ void HSP_Param_set_codon_hsp_threshold(HSP_Param *hsp_param,
     return;
     }
 
+/**/
+
+void HSP_Param_set_dna_hsp_dropoff(HSP_Param *hsp_param,
+                                   gint dna_hsp_dropoff){
+    if(hsp_param->match->type == Match_Type_DNA2DNA)
+        hsp_param->dropoff = dna_hsp_dropoff;
+    return;
+    }
+
+void HSP_Param_set_protein_hsp_dropoff(HSP_Param *hsp_param,
+                                       gint protein_hsp_dropoff){
+    if((hsp_param->match->type == Match_Type_PROTEIN2PROTEIN)
+    || (hsp_param->match->type == Match_Type_PROTEIN2DNA)
+    || (hsp_param->match->type == Match_Type_DNA2PROTEIN))
+        hsp_param->dropoff  = protein_hsp_dropoff;
+    return;
+    }
+
+void HSP_Param_set_codon_hsp_dropoff(HSP_Param *hsp_param,
+                                     gint codon_hsp_dropoff){
+    if(hsp_param->match->type == Match_Type_CODON2CODON)
+        hsp_param->dropoff = codon_hsp_dropoff;
+    return;
+    }
+
+/**/
+
 void HSP_Param_set_hsp_threshold(HSP_Param *hsp_param,
                                  gint hsp_threshold){
     hsp_param->threshold = hsp_threshold;
@@ -189,18 +218,12 @@ HSP_Param *HSP_Param_create(Match *match, gboolean use_horizon){
     if(hsp_param->has->use_wordhood_dropoff && (!hsp_param->wordlimit)){
         hsp_param->wordhood = NULL;
     } else {
-        if(match->type == Match_Type_CODON2CODON){
-            wha = WordHood_Alphabet_create_from_CodonSubmat(
-                                            match->mas->codon_submat, FALSE);
-        } else {
-            submat = (match->type == Match_Type_DNA2DNA)
-                   ? match->mas->dna_submat
-                   : match->mas->protein_submat;
-            wha = WordHood_Alphabet_create_from_Submat(
-                    (gchar*)match->comparison_alphabet->member,
-                    (gchar*)match->comparison_alphabet->member, submat, FALSE);
-            Submat_destroy(submat);
-            }
+        submat = (match->type == Match_Type_DNA2DNA)
+               ? match->mas->dna_submat
+               : match->mas->protein_submat;
+        wha = WordHood_Alphabet_create_from_Submat(
+                (gchar*)match->comparison_alphabet->member,
+                (gchar*)match->comparison_alphabet->member, submat, FALSE);
         g_assert(wha);
         hsp_param->wordhood = WordHood_create(wha, hsp_param->wordlimit,
                                           hsp_param->has->use_wordhood_dropoff);
@@ -1146,7 +1169,7 @@ void HSPset_filter_ungapped(HSPset *hsp_set){
 
 /**/
 
-#define HSPset_SList_PAGE_BIT_WIDTH 12
+#define HSPset_SList_PAGE_BIT_WIDTH 10
 #define HSPset_SList_PAGE_SIZE (1 << HSPset_SList_PAGE_BIT_WIDTH)
 
 RecycleBin *HSPset_SList_RecycleBin_create(void){
@@ -1164,23 +1187,93 @@ HSPset_SList_Node *HSPset_SList_append(RecycleBin *recycle_bin,
     return node;
     }
 
+#if 0
+typedef struct {
+                 gint      page_alloc;
+                 gint      page_total;
+    HSPset_SList_Node    **diag_page_list;
+                 gint  ****horizon;
+                 gint     *page_used;
+                 gint      page_used_total;
+} HSPset_SeedData;
+
+static HSPset_SeedData *HSPset_SeedData_create(HSP_Param *hsp_param,
+                                               gint target_len){
+    register HSPset_SeedData *seed_data = g_new(HSPset_SeedData, 1);
+    seed_data->page_total = (target_len
+                          >> HSPset_SList_PAGE_BIT_WIDTH) + 1;
+    seed_data->page_alloc = seed_data->page_total;
+    seed_data->diag_page_list = g_new0(HSPset_SList_Node*, seed_data->page_total);
+    seed_data->page_used = g_new(gint, seed_data->page_total);
+    seed_data->horizon = (gint****)Matrix4d_create(
+                           2 + ((hsp_param->seed_repeat > 1)?1:0),
+                           HSPset_SList_PAGE_SIZE,
+                           hsp_param->match->query->advance,
+                           hsp_param->match->target->advance,
+                           sizeof(gint));
+    seed_data->page_used_total = 0;
+    return seed_data;
+    }
+
+static void HSPset_SeedData_destroy(HSPset_SeedData *seed_data){
+    g_free(seed_data->diag_page_list);
+    g_free(seed_data->page_used);
+    g_free(seed_data->horizon);
+    g_free(seed_data);
+    return;
+    }
+
+static void HSPset_SeedData_set_target_len(HSPset_SeedData *seed_data,
+                                           HSPset *hsp_set){
+    register gint new_page_total = (hsp_set->target->len
+                                    >> HSPset_SList_PAGE_BIT_WIDTH) + 1;
+    register gint i, a, b, c, d;
+    seed_data->page_total = new_page_total;
+    if(seed_data->page_alloc < new_page_total){
+        seed_data->page_alloc = seed_data->page_total;
+        g_free(seed_data->diag_page_list);
+        seed_data->diag_page_list = g_new(HSPset_SList_Node*,
+                                          seed_data->page_total);
+        g_free(seed_data->page_used);
+        seed_data->page_used = g_new(gint, seed_data->page_total);
+        }
+    /* Clear diag_page_list */
+    for(i = 0; i < seed_data->page_total; i++)
+        seed_data->diag_page_list[i] = 0;
+    /* Clear horizon */
+    for(a = 2 + ((hsp_set->param->seed_repeat > 1)?1:0); a >= 0; a--)
+        for(b = HSPset_SList_PAGE_SIZE; b >= 0; b--)
+            for(c = hsp_set->param->match->query->advance; c >= 0; c--)
+                for(d = hsp_set->param->match->target->advance; d >= 0; d--)
+                    seed_data->horizon[a][b][c][d] = 0;
+    seed_data->page_used_total = 0;
+    return;
+    }
+#endif /* 0 */
+
 void HSPset_seed_all_qy_sorted(HSPset *hsp_set, HSPset_SList_Node *seed_list){
     register gint page_total = (hsp_set->target->len
                                 >> HSPset_SList_PAGE_BIT_WIDTH) + 1;
     register HSPset_SList_Node **diag_page_list
         = g_new0(HSPset_SList_Node*, page_total);
-    register gint i, page, diag_pos, query_frame, target_frame,
-                   section_pos, page_pos;
-    register HSPset_SList_Node *seed;
     register gint *page_used = g_new(gint, page_total);
-    register gint page_used_total = 0;
     register gint ****horizon = (gint****)Matrix4d_create(
                            2 + ((hsp_set->param->seed_repeat > 1)?1:0),
                            HSPset_SList_PAGE_SIZE,
                            hsp_set->param->match->query->advance,
                            hsp_set->param->match->target->advance,
                            sizeof(gint));
+    /*
+    register HSPset_SeedData *seed_data = HSPset_SeedData_create(hsp_set->param,
+                                                           hsp_set->target->len);
+    */
+    register gint i, page, diag_pos, query_frame, target_frame,
+                   section_pos, page_pos;
+    register HSPset_SList_Node *seed;
+    register gint page_used_total = 0;
     HSP nascent_hsp;
+    /* g_message("[%s] with [%d]", __FUNCTION__, hsp_set->target->len); */
+    /* HSPset_SeedData_set_target_len(seed_data, hsp_set); */
     /* Bin on diagonal into pages */
     while(seed_list){
         seed = seed_list;
@@ -1230,8 +1323,7 @@ void HSPset_seed_all_qy_sorted(HSPset *hsp_set, HSPset_SList_Node *seed_list){
                if(hsp_set->param->seed_repeat > 1)
                    horizon[2][page_pos][query_frame][target_frame] = 0;
                }
-            if(seed->query_pos
-            < horizon[0][page_pos][query_frame][target_frame])
+            if(seed->query_pos < horizon[0][page_pos][query_frame][target_frame])
                 continue;
             if(hsp_set->param->seed_repeat > 1){
                 if(++horizon[2][page_pos][query_frame][target_frame]
@@ -1272,9 +1364,9 @@ void HSPset_seed_all_qy_sorted(HSPset *hsp_set, HSPset_SList_Node *seed_list){
     g_free(page_used);
     g_free(horizon);
     HSPset_finalise(hsp_set);
+    /* HSPset_SeedData_destroy(seed_data); */
     return;
     }
-/* FIXME: move all allocs outside this function */
 /* horizon[horizon][mailbox][seed_repeat]
  *        [page_size][qadv][tadv]
  */
