@@ -156,12 +156,21 @@ HSP_Param *HSP_Param_share(HSP_Param *hsp_param);
 HSP_Param *HSP_Param_swap(HSP_Param *hsp_param);
 
      void  HSP_Param_set_wordlen(HSP_Param *hsp_param, gint wordlen);
+     /**/
      void  HSP_Param_set_dna_hsp_threshold(HSP_Param *hsp_param,
                                            gint dna_hsp_threshold);
      void  HSP_Param_set_protein_hsp_threshold(HSP_Param *hsp_param,
                                                gint dna_hsp_threshold);
      void  HSP_Param_set_codon_hsp_threshold(HSP_Param *hsp_param,
                                              gint dna_hsp_threshold);
+     /**/
+     void  HSP_Param_set_dna_hsp_dropoff(HSP_Param *hsp_param,
+                                         gint dna_hsp_dropoff);
+     void  HSP_Param_set_protein_hsp_dropoff(HSP_Param *hsp_param,
+                                             gint dna_hsp_dropoff);
+     void  HSP_Param_set_codon_hsp_dropoff(HSP_Param *hsp_param,
+                                           gint dna_hsp_dropoff);
+     /**/
      void  HSP_Param_set_hsp_threshold(HSP_Param *hsp_param,
                                        gint hsp_threshold);
      void  HSP_Param_set_seed_repeat(HSP_Param *hsp_param,
