@@ -41,10 +41,6 @@ Match_ArgumentSet *Match_ArgumentSet_create(Argument *arg){
         mas.translate = Translate_create(FALSE);
         /* FIXME this should be freed somewhere */
         }
-    if(!mas.codon_submat){
-        mas.codon_submat = CodonSubmat_create();
-        /* FIXME this should be freed somewhere */
-        }
     if(arg){
         as = ArgumentSet_create("Symbol Comparison Options");
         ArgumentSet_add_option(as, 0, "softmaskquery", NULL,
@@ -198,6 +194,7 @@ static Match_Score Match_3_translate_self_func(Match_Strand *strand,
                          symbol, symbol);
     }
 
+#if 0
 static Match_Score Match_3_codon_self_func(Match_Strand *strand,
                                            Sequence *seq, guint pos){
     register gint codon;
@@ -209,6 +206,7 @@ static Match_Score Match_3_codon_self_func(Match_Strand *strand,
     return CodonSubmat_lookup_codon(strand->match->mas->codon_submat,
                                     codon, codon);
     }
+#endif /* 0 */
 
 static gboolean Match_3_mask_func(Match_Strand *strand,
                                   Sequence *seq, guint pos){
@@ -483,6 +481,7 @@ static void Match_3_1_display_func(Match *match,
 
 /**/
 
+#if 0
 static Match_Score Match_3_3_split_score_func(Match *match,
                               Sequence *query, Sequence *target,
                               guint qp1, guint qp2, guint qp3,
@@ -502,6 +501,32 @@ static Match_Score Match_3_3_split_score_func(Match *match,
                 Sequence_get_symbol(target, tp2),
                 Sequence_get_symbol(target, tp3));
     }
+#endif /* 0 */
+
+/* FIXME: temp */
+static Match_Score Match_3_3_split_score_func(Match *match,
+                              Sequence *query, Sequence *target,
+                              guint qp1, guint qp2, guint qp3,
+                              guint tp1, guint tp2, guint tp3){
+    register gchar query_symbol, target_symbol;
+    if(query->annotation)
+        if(query->alphabet->type == Alphabet_Type_DNA)
+            if((qp1 < query->annotation->cds_start)
+            || (qp1 >= (query->annotation->cds_start
+                       +query->annotation->cds_length))
+            || ((qp1 % 3) != (query->annotation->cds_start % 3)))
+                return MATCH_IMPOSSIBLY_LOW_SCORE;
+    query_symbol = Translate_base(match->mas->translate,
+                       Sequence_get_symbol(query, qp1),
+                       Sequence_get_symbol(query, qp2),
+                       Sequence_get_symbol(query, qp3));
+    target_symbol = Translate_base(match->mas->translate,
+                       Sequence_get_symbol(target, tp1),
+                       Sequence_get_symbol(target, tp2),
+                       Sequence_get_symbol(target, tp3));
+    return Submat_lookup(match->mas->protein_submat,
+                query_symbol, target_symbol);
+    }
 
 static Match_Score Match_3_3_codon_score_func(Match *match,
                            Sequence *query, Sequence *target,
@@ -513,6 +538,7 @@ static Match_Score Match_3_3_codon_score_func(Match *match,
             target_pos, target_pos+1, target_pos+2);
     }
 
+#if 0
 static gchar *Match_3_3_score_macro(void){
     return "((ud->query->annotation)\n"
            " && (ud->query->alphabet->type == Alphabet_Type_DNA)\n"
@@ -533,6 +559,31 @@ static gchar *Match_3_3_score_macro(void){
  *        remove seq type check
  *        precompute cds_start % 3
  */
+#endif /* 0 */
+
+/* FIXME: temp */
+static gchar *Match_3_3_score_macro(void){
+    return "((ud->query->annotation)\n"
+           " && (ud->query->alphabet->type == Alphabet_Type_DNA)\n"
+           " && (  ((%QP) < ud->query->annotation->cds_start)\n"
+           "    || ((%QP) >= (ud->query->annotation->cds_start\n"
+           "               +ud->query->annotation->cds_length))\n"
+           "    || (((%QP) %% 3) != (ud->query->annotation->cds_start %% 3))))\n"
+           "?MATCH_IMPOSSIBLY_LOW_SCORE\n"
+           ":Submat_lookup(ud->mas->protein_submat,\n"
+           "               Translate_base(ud->mas->translate, \n"
+           "                  Sequence_get_symbol(ud->query, %QP),\n"
+           "                  Sequence_get_symbol(ud->query, %QP+1),\n"
+           "                  Sequence_get_symbol(ud->query, %QP+2)),\n"
+           "               Translate_base(ud->mas->translate, \n"
+           "                  Sequence_get_symbol(ud->target, %TP),\n"
+           "                  Sequence_get_symbol(ud->target, %TP+1),\n"
+           "                  Sequence_get_symbol(ud->target, %TP+2)))\n";
+    }
+/* FIXME: optimisations:
+ *        remove seq type check
+ *        precompute cds_start % 3
+ */
 
 static void Match_3_3_split_display_func(Match *match,
                               Sequence *query, Sequence *target,
@@ -618,9 +669,12 @@ static Match_Strand *Match_Strand_create(Alphabet_Type alphabet_type,
             strand->mask_func = Match_1_mask_func;
             break;
         case 3:
+            /*
             strand->self_func = is_translated
                               ? Match_3_translate_self_func
                               : Match_3_codon_self_func;
+                              */
+            strand->self_func = Match_3_translate_self_func; /* FIXME: temp */
             strand->mask_func = Match_3_mask_func;
             break;
         default:
@@ -701,18 +755,20 @@ static Match *Match_create_without_mirror(Match_Type type){
             match->split_display_func = Match_invalid_split_display_func;
             break;
         case Match_Type_CODON2CODON:
+            /* FIXME: should do this for mixed models only ?? */
+#if 0
             g_assert(match->mas->dna_submat);
             g_assert(match->mas->codon_submat);
-            /* FIXME: should do this for mixed models only ?? */
             CodonSubmat_add_nucleic(match->mas->codon_submat,
                                     match->mas->dna_submat);
+#endif /* 0 */
             /**/
             match->query = Match_Strand_create(Alphabet_Type_DNA,
                      match->mas->softmasked_query, FALSE, 3, match);
             match->target = Match_Strand_create(Alphabet_Type_DNA,
                      match->mas->softmasked_target, FALSE, 3, match);
             match->comparison_alphabet
-                = Alphabet_create(Alphabet_Type_DNA,
+                = Alphabet_create(Alphabet_Type_PROTEIN,
                                   softmask_comparison);
             match->score_func = Match_3_3_codon_score_func;
             match->display_func = Match_3_3_display_func;
@@ -832,9 +888,8 @@ Match_Score Match_max_score(Match *match){
         case Match_Type_PROTEIN2PROTEIN:
         case Match_Type_PROTEIN2DNA:
         case Match_Type_DNA2PROTEIN:
-            return Submat_max_score(match->mas->protein_submat);
         case Match_Type_CODON2CODON:
-            return CodonSubmat_max_score(match->mas->codon_submat);
+            return Submat_max_score(match->mas->protein_submat);
         default:
             g_error("Unknown match type [%d]", match->type);
         }
