@@ -34,7 +34,7 @@ typedef struct {
      gboolean  softmasked_target;
        Submat *dna_submat;
        Submat *protein_submat;
-  CodonSubmat *codon_submat;
+  /* CodonSubmat *codon_submat; */
     Translate *translate;
 } Match_ArgumentSet;
 
