@@ -3,11 +3,11 @@
 *  Library for PCR simulation                                    *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2006.  All Rights Reserved.                *
+*  Copyright (C) 2000-2008.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
-*  GNU Lesser General Public License. See the file COPYING       *
-*  or http://www.fsf.org/copyleft/lesser.html for details        *
+*  GNU General Public License, version 3. See the file COPYING   *
+*  or http://www.gnu.org/licenses/gpl.txt for details            *
 *                                                                *
 *  If you use this code, please keep this notice intact.         *
 *                                                                *
@@ -253,6 +253,7 @@ static PCR_Primer *PCR_Primer_create(PCR_Experiment *pcr_experiment,
     g_strup(pcr_primer->revcomp);
     pcr_primer->probe_list = g_ptr_array_new();
     WordHood_traverse(pcr_experiment->pcr->wordhood,
+                      PCR_Primer_Wordhood_traverse_func,
                       pcr_primer->forward,
                       pcr_primer->probe_len,
                       pcr_primer);
@@ -349,8 +350,7 @@ PCR *PCR_create(PCR_ReportFunc report_func, gpointer user_data,
     pcr->is_prepared = FALSE;
     wha = WordHood_Alphabet_create_from_Submat(
                             "GARTKWDCSMVYBHN", "ACGT", submat, FALSE);
-    pcr->wordhood = WordHood_create(wha, pcr->mismatch_threshold, TRUE,
-                                    PCR_Primer_Wordhood_traverse_func);
+    pcr->wordhood = WordHood_create(wha, pcr->mismatch_threshold, TRUE);
     WordHood_Alphabet_destroy(wha);
     pcr->experiment_memory_usage = 0;
     pcr->sensor_count = 0;
