@@ -217,9 +217,26 @@ static void WordHood_set_word(WordHood *wh, gchar *word, gint len,
     return;
     }
 
+static void WordHood_Alphabet_info(WordHood_Alphabet *wha){
+    g_message("WordHood_Alphabet:\n"
+     "         ref_count [%d]\n"
+     "         advance [%d]\n"
+     "         member_list->len [%d]\n"
+     "--\n",
+         wha->ref_count,
+         wha->advance,
+         wha->member_list->len);
+    return;
+    }
+
 void WordHood_info(WordHood *wh){
+    WordHood_Alphabet_info(wh->wha);
     g_message("WordHood:\n"
-     "         --\n");
+     "         threshold [%d]\n"
+     "         use_dropoff [%s]\n"
+     "--\n",
+        wh->threshold,
+        wh->use_dropoff?"yes":"no");
     return;
     }
 
