@@ -23,13 +23,13 @@
 /**/
 
 #define DATASET_HEADER_MAGIC (('e' << 16)|('s' << 8)|('d'))
-#define DATASET_HEADER_VERSION 1
+#define DATASET_HEADER_VERSION 2
 
 static Dataset_Header *Dataset_Header_create(Alphabet_Type alphabet_type,
                                              gboolean softmask_input){
     register Dataset_Header *header = g_new0(Dataset_Header, 1);
     header->magic = DATASET_HEADER_MAGIC;
-    header->version = 1;
+    header->version = DATASET_HEADER_VERSION;
     header->type = ((alphabet_type == Alphabet_Type_DNA)?1:0)
                  | ((softmask_input?1:0) << 1);
     /* other fields are filled during seq parsing or Dataset_finalise */
@@ -41,7 +41,49 @@ static void Dataset_Header_destroy(Dataset_Header *header){
     return;
     }
 
+#if 0
+static void Dataset_Header_info(Dataset_Header *header){
+    g_message("Dataset_Header:\n"
+              "     magic             [%lld]\n"
+              "     version           [%lld]\n"
+              "     type              [%lld]\n"
+              "     line_length       [%lld]\n"
+              /**/
+              "     number_of_dbs     [%lld]\n"
+              "     max_db_len        [%lld]\n"
+              "     total_db_len      [%lld]\n"
+              /**/
+              "     number_of_seqs    [%lld]\n"
+              "     max_seq_len       [%lld]\n"
+              "     total_seq_len     [%lld]\n"
+              /**/
+              "     path_data_offset  [%lld]\n"
+              "     seq_data_offset   [%lld]\n"
+              "     seq_info_offset   [%lld]\n"
+              "     total_file_length [%lld]\n",
+              header->magic,
+              header->version,
+              header->type,
+              header->line_length,
+              /**/
+              header->number_of_dbs,
+              header->max_db_len,
+              header->total_db_len,
+              /**/
+              header->number_of_seqs,
+              header->max_seq_len,
+              header->total_seq_len,
+              /**/
+              header->path_data_offset,
+              header->seq_data_offset,
+              header->seq_info_offset,
+              header->total_file_length);
+    return;
+    }
+#endif /* 0 */
+
 static void Dataset_Header_write(Dataset_Header *header, FILE *fp){
+    /* Dataset_Header_info(header); */
     /**/
     BitArray_write_int(header->magic, fp);
     BitArray_write_int(header->version, fp);
@@ -57,8 +99,8 @@ static void Dataset_Header_write(Dataset_Header *header, FILE *fp){
     BitArray_write_int(header->total_seq_len, fp);
     /**/
     BitArray_write_int(header->path_data_offset, fp);
-    BitArray_write_int(header->seq_info_offset, fp);
     BitArray_write_int(header->seq_data_offset, fp);
+    BitArray_write_int(header->seq_info_offset, fp);
     BitArray_write_int(header->total_file_length, fp);
     /**/
     return;
@@ -85,10 +127,11 @@ static Dataset_Header *Dataset_Header_read(FILE *fp){
     header->total_seq_len = BitArray_read_int(fp);
     /**/
     header->path_data_offset = BitArray_read_int(fp);
-    header->seq_info_offset = BitArray_read_int(fp);
     header->seq_data_offset = BitArray_read_int(fp);
+    header->seq_info_offset = BitArray_read_int(fp);
     header->total_file_length = BitArray_read_int(fp);
     /**/
+    /* Dataset_Header_info(header); */
     return header;
     }
 
@@ -212,22 +255,27 @@ Dataset *Dataset_create(GPtrArray *path_list,
         path = path_list->pdata[i];
         path_data_size += (strlen(path) + 1);
         }
-    seq_info_size = dataset->seq_list->len
-                  * dataset->width->seq_data_item_size;
     seq_data_size = 0;
     for(i = 0; i < dataset->seq_list->len; i++){
         ds = dataset->seq_list->pdata[i];
         ds->pos = i;
         seq_data_size += (strlen(ds->id) + 1);
+        if(ds->def)
+            seq_data_size += (strlen(ds->def) + 1);
         }
+    seq_info_size = dataset->seq_list->len
+                  * dataset->width->seq_data_item_size;
     /**/
     dataset->header->path_data_offset = sizeof(Dataset_Header);
-    dataset->header->seq_info_offset = dataset->header->path_data_offset
+    dataset->header->seq_data_offset = dataset->header->path_data_offset
                                      + path_data_size;
-    dataset->header->seq_data_offset = dataset->header->seq_info_offset
-                                     + seq_info_size;
-    dataset->header->total_file_length = dataset->header->seq_data_offset
-                                       + seq_data_size;
+    dataset->header->seq_info_offset = dataset->header->seq_data_offset
+                                     + seq_data_size;
+    dataset->header->total_file_length = dataset->header->seq_info_offset
+                                       + seq_info_size;
+#ifdef USE_PTHREADS
+    pthread_mutex_init(&dataset->dataset_mutex, NULL);
+#endif /* USE_PTHREADS */
     return dataset;
     }
 
@@ -268,6 +316,9 @@ void Dataset_destroy(Dataset *dataset){
     if(dataset->width)
         Dataset_Width_destroy(dataset->width);
     Alphabet_destroy(dataset->alphabet);
+#ifdef USE_PTHREADS
+    pthread_mutex_destroy(&dataset->dataset_mutex);
+#endif /* USE_PTHREADS */
     g_free(dataset);
     return;
     }
@@ -327,24 +378,36 @@ static void Dataset_write_seq_data(Dataset *dataset, FILE *fp){
     }
 
 static void Dataset_read_seq_data(Dataset *dataset, FILE *fp){
-    register gint i, j;
+    register gint i, j = 0, ipos, dpos;
     register Dataset_Sequence *ds;
-    gchar buf[1024];
+    register gint len = dataset->header->seq_info_offset
+                      - dataset->header->seq_data_offset;
+    register gchar *buf = g_new(gchar, len+1);
+    if(!fread(buf, sizeof(gchar), len, fp))
+        g_error("Problem reading seq data");
     for(i = 0; i < dataset->header->number_of_seqs; i++){
         ds = g_new0(Dataset_Sequence, 1);
         g_ptr_array_add(dataset->seq_list, ds);
-        if(!fgets(buf, 1024, fp))
-            g_error("Problem parsing seq data");
-        for(j = 0; buf[j]; j++){
-            if(buf[j] == ' '){
-                ds->id = g_strndup(buf, j);
-                ds->def = g_strndup(buf+j+1, strlen(buf+j)-2);
+        ipos = j;
+        dpos = -1;
+        while(buf[j]){
+            if((buf[j] == ' ') && (dpos == -1)){
+                buf[j] = '\0';
+                dpos = j+1;
+                }
+            if(buf[j] == '\n'){
+                buf[j] = '\0';
+                ds->id = g_strdup(buf+ipos);
+                if(dpos != -1)
+                    ds->def = g_strdup(buf+dpos);
+                j++;
                 break;
                 }
+            j++;
             }
-        if(!ds->id)
-            ds->id = g_strndup(buf, strlen(buf)-1);
+        g_assert(ds->id);
         }
+    g_free(buf);
     return;
     }
 
@@ -428,12 +491,15 @@ Dataset *Dataset_read(gchar *path){
                            ((dataset->header->type&1)
                             ?Alphabet_Type_DNA
                             :Alphabet_Type_PROTEIN),
-                            (dataset->header->type&3));
+                            (dataset->header->type&2));
     dataset->width = Dataset_Width_create(dataset->header);
     dataset->seq_list = g_ptr_array_new();
     Dataset_read_path_data(dataset, fp);
     Dataset_read_seq_data(dataset, fp);
     Dataset_read_seq_info(dataset, fp);
+#ifdef USE_PTHREADS
+    pthread_mutex_init(&dataset->dataset_mutex, NULL);
+#endif /* USE_PTHREADS */
     fclose(fp);
     return dataset;
     }
@@ -453,7 +519,7 @@ gint Dataset_lookup_id(Dataset *dataset, gchar *id){
     }
 
 Sequence *Dataset_get_sequence(Dataset *dataset, gint dataset_pos){
-    register Sequence *seq;
+    register Sequence *seq = NULL;
     register Dataset_Sequence *ds;
     register gchar *def;
     register SparseCache *cache;
@@ -461,17 +527,26 @@ Sequence *Dataset_get_sequence(Dataset *dataset, gint dataset_pos){
     g_assert(dataset_pos >= 0);
     g_assert(dataset_pos < dataset->seq_list->len);
     ds = dataset->seq_list->pdata[dataset_pos];
+#ifdef USE_PTHREADS
+    pthread_mutex_lock(&dataset->dataset_mutex);
+#endif /* USE_PTHREADS */
     if(ds->cache_seq)
-        return Sequence_share(ds->cache_seq);
+        seq = Sequence_share(ds->cache_seq);
+#ifdef USE_PTHREADS
+    pthread_mutex_unlock(&dataset->dataset_mutex);
+#endif /* USE_PTHREADS */
+    if(seq)
+        return seq;
+    def = FastaDB_Key_get_def(ds->key);
     strand = (dataset->alphabet->type == Alphabet_Type_DNA)
            ? Sequence_Strand_FORWARD
            : Sequence_Strand_UNKNOWN;
-    def = FastaDB_Key_get_def(ds->key);
     cache = FastaDB_Key_get_SparseCache(ds->key);
     seq = Sequence_create_extmem(ds->id, def, ds->key->length,
                                  strand, dataset->alphabet, cache);
     g_free(def);
     SparseCache_destroy(cache);
+    g_assert(Sequence_checksum(seq) == ds->gcg_checksum); /* FIXME: slow */
     return seq;
     }
 /* FIXME: should keep a single compound file for whole dataset,
@@ -500,7 +575,6 @@ void Dataset_preload_seqs(Dataset *dataset){
     register Dataset_Sequence *ds;
     register Dataset_Sequence **ds_list = g_new(Dataset_Sequence*,
                                                 dataset->header->number_of_seqs);
-    g_message("Preloading sequences");
     for(i = 0; i < dataset->header->number_of_seqs; i++){
         ds = dataset->seq_list->pdata[i];
         ds->cache_seq = Dataset_get_sequence(dataset, i);
