@@ -23,6 +23,10 @@ extern "C" {
 #include <stdio.h>
 #include <glib.h>
 
+#ifdef USE_PTHREADS
+#include <pthread.h>
+#endif /* USE_PTHREADS */
+
 #include "alphabet.h"
 #include "sequence.h"
 #include "fastadb.h"
@@ -55,8 +59,8 @@ typedef struct {
     guint64 total_seq_len;
     /**/
     guint64 path_data_offset;
-    guint64 seq_info_offset;
     guint64 seq_data_offset;
+    guint64 seq_info_offset;
     guint64 total_file_length;
 } Dataset_Header;
 
@@ -83,6 +87,9 @@ typedef struct {
      Dataset_Width *width;
          GPtrArray *seq_list;  /* containing Dataset_Sequence objects */
            FastaDB *fdb;
+#ifdef USE_PTHREADS
+    pthread_mutex_t dataset_mutex;
+#endif /* USE_PTHREADS */
 } Dataset;
 
 Dataset *Dataset_create(GPtrArray *path_list,
