@@ -319,14 +319,16 @@ FastaDB_Seq *FastaDB_next(FastaDB *fdb, FastaDB_Mask mask){
     if(mask & FastaDB_Mask_ID){ /* Record ID */
         id_pos = 0;
         while((ch = CompoundFile_getc(fdb->cf)) != EOF){
-            if((ch == '\n') || (ch == ' ') || (ch == '\t'))
+            /* if((ch == '\n') || (ch == ' ') || (ch == '\t')) */
+            if(isspace(ch))
                 break;
             FastaDB_putc(fdb, ch);
             }
         FastaDB_putc(fdb, '\0');
     } else { /* Skip ID */
         while((ch = CompoundFile_getc(fdb->cf)) != EOF){
-            if((ch == '\n') || (ch == ' ') || (ch == '\t'))
+            /* if((ch == '\n') || (ch == ' ') || (ch == '\t')) */
+            if(isspace(ch))
                 break;
             }
         }
@@ -449,8 +451,8 @@ void FastaDB_Key_destroy(FastaDB_Key *fdbk){
 
 FastaDB_Key *FastaDB_Seq_get_key(FastaDB_Seq *fdbs){
     register FastaDB_Key *fdbk;
-    register gint seq_offset = 2 + strlen(fdbs->seq->id)
-                             + (fdbs->seq->def ? strlen(fdbs->seq->def) : 0);
+    register gint seq_offset = strlen(fdbs->seq->id) + 2
+                             + (fdbs->seq->def ? (strlen(fdbs->seq->def)+1) : 0);
     g_assert(fdbs);
     fdbk = FastaDB_Key_create(fdbs->source, fdbs->location,
                               fdbs->seq->strand, seq_offset,
@@ -495,18 +497,28 @@ gchar *FastaDB_Key_get_def(FastaDB_Key *fdbk){
     register gint ch;
     register gchar *def = NULL;
     register GString *s;
+#ifdef USE_PTHREADS
+    pthread_mutex_lock(&fdbk->source->cf->compoundfile_mutex);
+#endif /* USE_PTHREADS */
     CompoundFile_Location_seek(fdbk->location);
     while((ch = CompoundFile_getc(fdbk->source->cf)) != EOF)
         if((ch == '\n') || (ch == ' ') || (ch == '\t'))
             break;
-    if(ch == '\n')
+    if(ch == '\n'){
+#ifdef USE_PTHREADS
+        pthread_mutex_unlock(&fdbk->source->cf->compoundfile_mutex);
+#endif /* USE_PTHREADS */
         return NULL;
+        }
     s = g_string_sized_new(64);
     while((ch = CompoundFile_getc(fdbk->source->cf)) != EOF){
         if(ch == '\n')
             break;
         s = g_string_append_c(s, ch);
         }
+#ifdef USE_PTHREADS
+    pthread_mutex_unlock(&fdbk->source->cf->compoundfile_mutex);
+#endif /* USE_PTHREADS */
     def = s->str;
     g_string_free(s, FALSE);
     return def;
@@ -519,16 +531,46 @@ static gpointer FastaDB_SparseCache_get_func_8_BIT(gint pos, gpointer page_data,
     return GINT_TO_POINTER((gint)((gchar*)page_data)[pos]);
     }
 
-static gpointer FastaDB_SparseCache_get_func_0_BIT_LC(gint pos, gpointer page_data,
+static gint FastaDB_SparseCache_copy_func_8_BIT(gint start, gint length,
+                                                gchar *dst, gpointer page_data,
+                                                gpointer user_data){
+    register gint i;
+    register gchar *str = page_data;
+    for(i = 0; i < length; i++)
+        dst[i] = str[start+i];
+    return length;
+    }
+
+static gpointer FastaDB_SparseCache_get_func_0_BIT_LC(gint pos,
+                                                      gpointer page_data,
                                                       gpointer user_data){
     return GINT_TO_POINTER((gint)'n');
     }
 
-static gpointer FastaDB_SparseCache_get_func_0_BIT_UC(gint pos, gpointer page_data,
+static gint FastaDB_SparseCache_copy_func_0_BIT_LC(gint start, gint length,
+                                                   gchar *dst, gpointer page_data,
+                                                   gpointer user_data){
+    register gint i;
+    for(i = 0; i < length; i++)
+        dst[i] = 'n';
+    return length;
+    }
+
+static gpointer FastaDB_SparseCache_get_func_0_BIT_UC(gint pos,
+                                                      gpointer page_data,
                                                       gpointer user_data){
     return GINT_TO_POINTER((gint)'N');
     }
 
+static gint FastaDB_SparseCache_copy_func_0_BIT_UC(gint start, gint length,
+                                                   gchar *dst, gpointer page_data,
+                                                   gpointer user_data){
+    register gint i;
+    for(i = 0; i < length; i++)
+        dst[i] = 'N';
+    return length;
+    }
+
 typedef enum {
     FastaDB_SparseCache_Mask_0_BIT_LC, /* [n] */
     FastaDB_SparseCache_Mask_0_BIT_UC, /* [N] */
@@ -566,15 +608,87 @@ static void FastaDB_SparseCache_fill_index(gint *index, gchar *alphabet){
     return;
     }
 
-static gpointer FastaDB_SparseCache_get_func_4_BIT(gint pos, gpointer page_data,
+static gpointer FastaDB_SparseCache_get_func_4_BIT(gint pos,
+                                                   gpointer page_data,
                                                    gpointer user_data){
     register gint ch = ((gchar*)page_data)[(pos >> 1)],
                   num = (ch >> ((pos & 1) << 2)) & 15;
-    static gchar *alphabet = "ACGTNacgtn";
+    static gchar *alphabet = "ACGTNacgtn------";
     return GINT_TO_POINTER((gint)alphabet[num]);
     }
 
-static void FastaDB_SparseCache_compress_4bit(SparseCache_Page *page, gint len){
+static gint FastaDB_SparseCache_copy_func_4_BIT(gint start, gint length,
+                                                gchar *dst, gpointer page_data,
+                                                gpointer user_data){
+#ifndef G_DISABLE_ASSERT
+    register gint i;
+#endif /* G_DISABLE_ASSERT */
+    register gint pos = 0, end = length;
+    register gchar *str = page_data, *word;
+    static gchar *table[256] = {
+        "AA", "CA", "GA", "TA", "NA", "aA", "cA", "gA",
+        "tA", "nA", "-A", "-A", "-A", "-A", "-A", "-A",
+        "AC", "CC", "GC", "TC", "NC", "aC", "cC", "gC",
+        "tC", "nC", "-C", "-C", "-C", "-C", "-C", "-C",
+        "AG", "CG", "GG", "TG", "NG", "aG", "cG", "gG",
+        "tG", "nG", "-G", "-G", "-G", "-G", "-G", "-G",
+        "AT", "CT", "GT", "TT", "NT", "aT", "cT", "gT",
+        "tT", "nT", "-T", "-T", "-T", "-T", "-T", "-T",
+        "AN", "CN", "GN", "TN", "NN", "aN", "cN", "gN",
+        "tN", "nN", "-N", "-N", "-N", "-N", "-N", "-N",
+        "Aa", "Ca", "Ga", "Ta", "Na", "aa", "ca", "ga",
+        "ta", "na", "-a", "-a", "-a", "-a", "-a", "-a",
+        "Ac", "Cc", "Gc", "Tc", "Nc", "ac", "cc", "gc",
+        "tc", "nc", "-c", "-c", "-c", "-c", "-c", "-c",
+        "Ag", "Cg", "Gg", "Tg", "Ng", "ag", "cg", "gg",
+        "tg", "ng", "-g", "-g", "-g", "-g", "-g", "-g",
+        "At", "Ct", "Gt", "Tt", "Nt", "at", "ct", "gt",
+        "tt", "nt", "-t", "-t", "-t", "-t", "-t", "-t",
+        "An", "Cn", "Gn", "Tn", "Nn", "an", "cn", "gn",
+        "tn", "nn", "-n", "-n", "-n", "-n", "-n", "-n",
+        "A-", "C-", "G-", "T-", "N-", "a-", "c-", "g-",
+        "t-", "n-", "--", "--", "--", "--", "--", "--",
+        "A-", "C-", "G-", "T-", "N-", "a-", "c-", "g-",
+        "t-", "n-", "--", "--", "--", "--", "--", "--",
+        "A-", "C-", "G-", "T-", "N-", "a-", "c-", "g-",
+        "t-", "n-", "--", "--", "--", "--", "--", "--",
+        "A-", "C-", "G-", "T-", "N-", "a-", "c-", "g-",
+        "t-", "n-", "--", "--", "--", "--", "--", "--",
+        "A-", "C-", "G-", "T-", "N-", "a-", "c-", "g-",
+        "t-", "n-", "--", "--", "--", "--", "--", "--",
+        "A-", "C-", "G-", "T-", "N-", "a-", "c-", "g-",
+        "t-", "n-", "--", "--", "--", "--", "--", "--"
+        };
+    g_assert(length);
+    if(start&1){ /* If odd start, get first base */
+        dst[0] = GPOINTER_TO_INT(FastaDB_SparseCache_get_func_4_BIT(start,
+                                                     page_data, user_data));
+        pos = 1;
+        }
+    if((length > 2) && ((start+length)&1)){ /* If odd end, get last base */
+        dst[length-1] = GPOINTER_TO_INT(FastaDB_SparseCache_get_func_4_BIT(
+                                       start+length-1, page_data, user_data));
+        end--;
+        }
+    while(pos < end){
+        word = table[(guchar)str[(start+pos)>>1]];
+        g_assert(word[0] != '-');
+        g_assert(word[1] != '-');
+        dst[pos++] = word[0];
+        dst[pos++] = word[1];
+        }
+#ifndef G_DISABLE_ASSERT
+    for(i = 0; i < length; i++){
+        g_assert(dst[i]
+                 == GPOINTER_TO_INT(FastaDB_SparseCache_get_func_4_BIT(
+                                    start+i, page_data, user_data)));
+        }
+#endif /* G_DISABLE_ASSERT */
+    return length;
+    }
+
+static void FastaDB_SparseCache_compress_4bit(SparseCache_Page *page,
+                                              gint len){
     register gint i;
     register gchar *seq =  page->data;
     static gint index[ALPHABET_SIZE];
@@ -586,23 +700,99 @@ static void FastaDB_SparseCache_compress_4bit(SparseCache_Page *page, gint len){
     page->data = g_new0(gchar, (len >> 1)+1);
     page->data_size = sizeof(gchar)*((len >> 1)+1);
     for(i = 0; i < len; i++)
-        ((gchar*)page->data)[(i >> 1)] |= (index[(guchar)seq[i]] << ((i&1) << 2));
+        ((gchar*)page->data)[(i >> 1)]
+        |= (index[(guchar)seq[i]] << ((i&1) << 2));
     g_free(seq);
     page->get_func = FastaDB_SparseCache_get_func_4_BIT;
+    page->copy_func = FastaDB_SparseCache_copy_func_4_BIT;
     return;
     }
 
 /**/
 
-static gpointer FastaDB_SparseCache_get_func_2_BIT_LC(gint pos, gpointer page_data,
-                                                   gpointer user_data){
+static gpointer FastaDB_SparseCache_get_func_2_BIT_LC(gint pos,
+                                                      gpointer page_data,
+                                                      gpointer user_data){
     register gint ch = ((gchar*)page_data)[(pos >> 2)],
                   num = (ch >> ((pos & 3) << 1)) & 3;
     static gchar *alphabet = "acgt";
     return GINT_TO_POINTER((gint)alphabet[num]);
     }
 
-static void FastaDB_SparseCache_compress_2bit_lc(SparseCache_Page *page, gint len){
+static gint FastaDB_SparseCache_copy_func_2_BIT_LC(gint start, gint length,
+                                                   gchar *dst, gpointer page_data,
+                                                   gpointer user_data){
+#ifndef G_DISABLE_ASSERT
+    register gint i;
+#endif /* G_DISABLE_ASSERT */
+    register gint pos = 0, end = length;
+    register gchar *str = page_data, *word;
+    gchar *table[256] = {
+        "aaaa", "caaa", "gaaa", "taaa", "acaa", "ccaa", "gcaa", "tcaa",
+        "agaa", "cgaa", "ggaa", "tgaa", "ataa", "ctaa", "gtaa", "ttaa",
+        "aaca", "caca", "gaca", "taca", "acca", "ccca", "gcca", "tcca",
+        "agca", "cgca", "ggca", "tgca", "atca", "ctca", "gtca", "ttca",
+        "aaga", "caga", "gaga", "taga", "acga", "ccga", "gcga", "tcga",
+        "agga", "cgga", "ggga", "tgga", "atga", "ctga", "gtga", "ttga",
+        "aata", "cata", "gata", "tata", "acta", "ccta", "gcta", "tcta",
+        "agta", "cgta", "ggta", "tgta", "atta", "ctta", "gtta", "ttta",
+        "aaac", "caac", "gaac", "taac", "acac", "ccac", "gcac", "tcac",
+        "agac", "cgac", "ggac", "tgac", "atac", "ctac", "gtac", "ttac",
+        "aacc", "cacc", "gacc", "tacc", "accc", "cccc", "gccc", "tccc",
+        "agcc", "cgcc", "ggcc", "tgcc", "atcc", "ctcc", "gtcc", "ttcc",
+        "aagc", "cagc", "gagc", "tagc", "acgc", "ccgc", "gcgc", "tcgc",
+        "aggc", "cggc", "gggc", "tggc", "atgc", "ctgc", "gtgc", "ttgc",
+        "aatc", "catc", "gatc", "tatc", "actc", "cctc", "gctc", "tctc",
+        "agtc", "cgtc", "ggtc", "tgtc", "attc", "cttc", "gttc", "tttc",
+        "aaag", "caag", "gaag", "taag", "acag", "ccag", "gcag", "tcag",
+        "agag", "cgag", "ggag", "tgag", "atag", "ctag", "gtag", "ttag",
+        "aacg", "cacg", "gacg", "tacg", "accg", "cccg", "gccg", "tccg",
+        "agcg", "cgcg", "ggcg", "tgcg", "atcg", "ctcg", "gtcg", "ttcg",
+        "aagg", "cagg", "gagg", "tagg", "acgg", "ccgg", "gcgg", "tcgg",
+        "aggg", "cggg", "gggg", "tggg", "atgg", "ctgg", "gtgg", "ttgg",
+        "aatg", "catg", "gatg", "tatg", "actg", "cctg", "gctg", "tctg",
+        "agtg", "cgtg", "ggtg", "tgtg", "attg", "cttg", "gttg", "tttg",
+        "aaat", "caat", "gaat", "taat", "acat", "ccat", "gcat", "tcat",
+        "agat", "cgat", "ggat", "tgat", "atat", "ctat", "gtat", "ttat",
+        "aact", "cact", "gact", "tact", "acct", "ccct", "gcct", "tcct",
+        "agct", "cgct", "ggct", "tgct", "atct", "ctct", "gtct", "ttct",
+        "aagt", "cagt", "gagt", "tagt", "acgt", "ccgt", "gcgt", "tcgt",
+        "aggt", "cggt", "gggt", "tggt", "atgt", "ctgt", "gtgt", "ttgt",
+        "aatt", "catt", "gatt", "tatt", "actt", "cctt", "gctt", "tctt",
+        "agtt", "cgtt", "ggtt", "tgtt", "attt", "cttt", "gttt", "tttt",
+        };
+    while((start+pos)&3){
+        dst[pos] = GPOINTER_TO_INT(FastaDB_SparseCache_get_func_2_BIT_LC(
+                                   start+pos, page_data, user_data));
+        pos++;
+        }
+    if(length > 4){
+        while(end&3){
+            dst[end-start-1]
+                = GPOINTER_TO_INT(FastaDB_SparseCache_get_func_2_BIT_LC(
+                                  end-1, page_data, user_data));
+            end--;
+            }
+        }
+    while(pos < end){
+        word = table[(guchar)str[(start+pos)>>2]];
+        dst[pos++] = word[0];
+        dst[pos++] = word[1];
+        dst[pos++] = word[2];
+        dst[pos++] = word[3];
+        }
+#ifndef G_DISABLE_ASSERT
+    for(i = 0; i < length; i++){
+        g_assert(dst[i]
+          == GPOINTER_TO_INT(FastaDB_SparseCache_get_func_2_BIT_LC(
+                             start+i, page_data, user_data)));
+        }
+#endif /* G_DISABLE_ASSERT */
+    return length;
+    }
+
+static void FastaDB_SparseCache_compress_2bit_lc(SparseCache_Page *page,
+                                                 gint len){
     register gint i;
     register gchar *seq =  page->data;
     static gint index[ALPHABET_SIZE];
@@ -617,18 +807,93 @@ static void FastaDB_SparseCache_compress_2bit_lc(SparseCache_Page *page, gint le
         ((gchar*)page->data)[(i >> 2)] |= (index[(guchar)seq[i]] << ((i&3) << 1));
     g_free(seq);
     page->get_func = FastaDB_SparseCache_get_func_2_BIT_LC;
+    page->copy_func = FastaDB_SparseCache_copy_func_2_BIT_LC;
     return;
     }
 
-static gpointer FastaDB_SparseCache_get_func_2_BIT_UC(gint pos, gpointer page_data,
-                                                   gpointer user_data){
+static gpointer FastaDB_SparseCache_get_func_2_BIT_UC(gint pos,
+                                                      gpointer page_data,
+                                                      gpointer user_data){
     register gint ch = ((gchar*)page_data)[(pos >> 2)],
                   num = (ch >> ((pos & 3) << 1)) & 3;
     static gchar *alphabet = "ACGT";
     return GINT_TO_POINTER((gint)alphabet[num]);
     }
 
-static void FastaDB_SparseCache_compress_2bit_uc(SparseCache_Page *page, gint len){
+static gint FastaDB_SparseCache_copy_func_2_BIT_UC(gint start, gint length,
+                                                   gchar *dst, gpointer page_data,
+                                                   gpointer user_data){
+#ifndef G_DISABLE_ASSERT
+    register gint i;
+#endif /* G_DISABLE_ASSERT */
+    register gint pos = 0, end = length;
+    register gchar *str = page_data, *word;
+    gchar *table[256] = {
+        "AAAA", "CAAA", "GAAA", "TAAA", "ACAA", "CCAA", "GCAA", "TCAA",
+        "AGAA", "CGAA", "GGAA", "TGAA", "ATAA", "CTAA", "GTAA", "TTAA",
+        "AACA", "CACA", "GACA", "TACA", "ACCA", "CCCA", "GCCA", "TCCA",
+        "AGCA", "CGCA", "GGCA", "TGCA", "ATCA", "CTCA", "GTCA", "TTCA",
+        "AAGA", "CAGA", "GAGA", "TAGA", "ACGA", "CCGA", "GCGA", "TCGA",
+        "AGGA", "CGGA", "GGGA", "TGGA", "ATGA", "CTGA", "GTGA", "TTGA",
+        "AATA", "CATA", "GATA", "TATA", "ACTA", "CCTA", "GCTA", "TCTA",
+        "AGTA", "CGTA", "GGTA", "TGTA", "ATTA", "CTTA", "GTTA", "TTTA",
+        "AAAC", "CAAC", "GAAC", "TAAC", "ACAC", "CCAC", "GCAC", "TCAC",
+        "AGAC", "CGAC", "GGAC", "TGAC", "ATAC", "CTAC", "GTAC", "TTAC",
+        "AACC", "CACC", "GACC", "TACC", "ACCC", "CCCC", "GCCC", "TCCC",
+        "AGCC", "CGCC", "GGCC", "TGCC", "ATCC", "CTCC", "GTCC", "TTCC",
+        "AAGC", "CAGC", "GAGC", "TAGC", "ACGC", "CCGC", "GCGC", "TCGC",
+        "AGGC", "CGGC", "GGGC", "TGGC", "ATGC", "CTGC", "GTGC", "TTGC",
+        "AATC", "CATC", "GATC", "TATC", "ACTC", "CCTC", "GCTC", "TCTC",
+        "AGTC", "CGTC", "GGTC", "TGTC", "ATTC", "CTTC", "GTTC", "TTTC",
+        "AAAG", "CAAG", "GAAG", "TAAG", "ACAG", "CCAG", "GCAG", "TCAG",
+        "AGAG", "CGAG", "GGAG", "TGAG", "ATAG", "CTAG", "GTAG", "TTAG",
+        "AACG", "CACG", "GACG", "TACG", "ACCG", "CCCG", "GCCG", "TCCG",
+        "AGCG", "CGCG", "GGCG", "TGCG", "ATCG", "CTCG", "GTCG", "TTCG",
+        "AAGG", "CAGG", "GAGG", "TAGG", "ACGG", "CCGG", "GCGG", "TCGG",
+        "AGGG", "CGGG", "GGGG", "TGGG", "ATGG", "CTGG", "GTGG", "TTGG",
+        "AATG", "CATG", "GATG", "TATG", "ACTG", "CCTG", "GCTG", "TCTG",
+        "AGTG", "CGTG", "GGTG", "TGTG", "ATTG", "CTTG", "GTTG", "TTTG",
+        "AAAT", "CAAT", "GAAT", "TAAT", "ACAT", "CCAT", "GCAT", "TCAT",
+        "AGAT", "CGAT", "GGAT", "TGAT", "ATAT", "CTAT", "GTAT", "TTAT",
+        "AACT", "CACT", "GACT", "TACT", "ACCT", "CCCT", "GCCT", "TCCT",
+        "AGCT", "CGCT", "GGCT", "TGCT", "ATCT", "CTCT", "GTCT", "TTCT",
+        "AAGT", "CAGT", "GAGT", "TAGT", "ACGT", "CCGT", "GCGT", "TCGT",
+        "AGGT", "CGGT", "GGGT", "TGGT", "ATGT", "CTGT", "GTGT", "TTGT",
+        "AATT", "CATT", "GATT", "TATT", "ACTT", "CCTT", "GCTT", "TCTT",
+        "AGTT", "CGTT", "GGTT", "TGTT", "ATTT", "CTTT", "GTTT", "TTTT",
+        };
+    while((start+pos)&3){
+        dst[pos] = GPOINTER_TO_INT(FastaDB_SparseCache_get_func_2_BIT_UC(
+                                   start+pos, page_data, user_data));
+        pos++;
+        }
+    if(length > 4){
+        while(end&3){
+            dst[end-start-1]
+                = GPOINTER_TO_INT(FastaDB_SparseCache_get_func_2_BIT_UC(end-1,
+                                              page_data, user_data));
+            end--;
+            }
+        }
+    while(pos < end){
+        word = table[(guchar)str[(start+pos)>>2]];
+        dst[pos++] = word[0];
+        dst[pos++] = word[1];
+        dst[pos++] = word[2];
+        dst[pos++] = word[3];
+        }
+#ifndef G_DISABLE_ASSERT
+    for(i = 0; i < length; i++){
+        g_assert(dst[i]
+          == GPOINTER_TO_INT(FastaDB_SparseCache_get_func_2_BIT_UC(
+                             start+i, page_data, user_data)));
+        }
+#endif /* G_DISABLE_ASSERT */
+    return length;
+    }
+
+static void FastaDB_SparseCache_compress_2bit_uc(SparseCache_Page *page,
+                                                 gint len){
     register gint i;
     register gchar *seq =  page->data;
     static gint index[ALPHABET_SIZE];
@@ -643,6 +908,7 @@ static void FastaDB_SparseCache_compress_2bit_uc(SparseCache_Page *page, gint le
         ((gchar*)page->data)[(i >> 2)] |= (index[(guchar)seq[i]] << ((i&3) << 1));
     g_free(seq);
     page->get_func = FastaDB_SparseCache_get_func_2_BIT_UC;
+    page->copy_func = FastaDB_SparseCache_copy_func_2_BIT_UC;
     return;
     }
 
@@ -659,8 +925,10 @@ void FastaDB_SparseCache_compress(SparseCache_Page *page, gint len){
         }
     for(i = 0; i < len; i++)
         mask |= mask_index[(guchar)seq[i]];
-    if(mask & (1 << FastaDB_SparseCache_Mask_8_BIT))
+    if(mask & (1 << FastaDB_SparseCache_Mask_8_BIT)){
+        page->copy_func = FastaDB_SparseCache_copy_func_8_BIT;
         return; /* Keep current 8 bit encoding */
+        }
     if(mask & (mask-1)){ /* more than one mask bit is set */
         FastaDB_SparseCache_compress_4bit(page, len);
         return;
@@ -678,6 +946,7 @@ void FastaDB_SparseCache_compress(SparseCache_Page *page, gint len){
         page->data = NULL;
         page->data_size = 0;
         page->get_func = FastaDB_SparseCache_get_func_0_BIT_LC;
+        page->copy_func = FastaDB_SparseCache_copy_func_0_BIT_LC;
         return;
         }
     if(mask & (1 << FastaDB_SparseCache_Mask_0_BIT_UC)){
@@ -685,6 +954,7 @@ void FastaDB_SparseCache_compress(SparseCache_Page *page, gint len){
         page->data = NULL;
         page->data_size = 0;
         page->get_func = FastaDB_SparseCache_get_func_0_BIT_UC;
+        page->copy_func = FastaDB_SparseCache_copy_func_0_BIT_UC;
         return;
         }
     g_error("Bad FastaDB_SparseCache compression mask");
@@ -697,20 +967,24 @@ static SparseCache_Page *FastaDB_SparseCache_fill_func(gint start,
     register FastaDB_Key *fdbk = user_data;
     register gint ch, pos = 0, len;
     register gint db_start, db_end;
+    if(fdbk->source->line_length < 1)
+        g_error("Unknown or irregular fasta line length");
     db_start = fdbk->seq_offset
              + start
-             + ((fdbk->source->line_length == -1)
-               ?0:(start / (fdbk->source->line_length+1)));
+             + (start / (fdbk->source->line_length));
     len = MIN(SparseCache_PAGE_SIZE, fdbk->length-start);
     db_end = db_start
            + len
-           + ((fdbk->source->line_length == -1)
-             ?0:(len / (fdbk->source->line_length+1)));
+           + (len / (fdbk->source->line_length));
+#ifdef USE_PTHREADS
+    pthread_mutex_lock(&fdbk->source->cf->compoundfile_mutex);
+#endif /* USE_PTHREADS */
     fdbk->location->pos += db_start;
     CompoundFile_Location_seek(fdbk->location);
     fdbk->location->pos -= db_start;
     page->data = g_new(gchar, len);
     page->get_func = FastaDB_SparseCache_get_func_8_BIT;
+    page->copy_func = NULL; /* FIXME: temp */
     page->data_size = sizeof(gchar)*len;
     do {
         ch = CompoundFile_getc(fdbk->source->cf);
@@ -719,6 +993,9 @@ static SparseCache_Page *FastaDB_SparseCache_fill_func(gint start,
         if(!isspace(ch))
             ((gchar*)page->data)[pos++] = ch;
     } while(pos < len);
+#ifdef USE_PTHREADS
+    pthread_mutex_unlock(&fdbk->source->cf->compoundfile_mutex);
+#endif /* USE_PTHREADS */
     FastaDB_SparseCache_compress(page, len);
     return page;
     }
