@@ -127,6 +127,9 @@ CompoundFile *CompoundFile_create(GPtrArray *path_list,
     if(sort_on_file_size)
         CompoundFile_sort(cf);
     CompoundFile_rewind(cf);
+#ifdef USE_PTHREADS
+    pthread_mutex_init(&cf->compoundfile_mutex, NULL);
+#endif /* USE_PTHREADS */
     return cf;
     }
 
@@ -145,6 +148,9 @@ void CompoundFile_destroy(CompoundFile *cf){
         CompoundFile_Location_destroy(cf->start_limit);
     if(cf->stop_limit)
         CompoundFile_Location_destroy(cf->stop_limit);
+#ifdef USE_PTHREADS
+    pthread_mutex_destroy(&cf->compoundfile_mutex);
+#endif /* USE_PTHREADS */
     g_free(cf);
     return;
     }
