@@ -26,6 +26,10 @@ extern "C" {
 #include <sys/types.h> /* For lseek() and off_t */
 #include <unistd.h>    /* For lseek() and off_t */
 
+#ifdef USE_PTHREADS
+#include <pthread.h>
+#endif /* USE_PTHREADS */
+
 #define COMPOUND_FILE_BUFFER_SIZE BUFSIZ
 /* Using BUFSIZ from stdio.h, although I'm not sure
  * why the stdio buffered IO with getc isn't just as
@@ -68,6 +72,9 @@ typedef struct {
                 CompoundFile_Pos  in_buffer_start;
     struct CompoundFile_Location *start_limit;
     struct CompoundFile_Location *stop_limit;
+#ifdef USE_PTHREADS
+                 pthread_mutex_t  compoundfile_mutex;
+#endif /* USE_PTHREADS */
 } CompoundFile;
 
 CompoundFile *CompoundFile_create(GPtrArray *path_list,
