@@ -20,6 +20,7 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/time.h>
+#include <netinet/tcp.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
@@ -35,12 +36,18 @@ static SocketConnection *SocketConnection_create(gchar *host,
                                                  gint  port){
     register SocketConnection *connection
      = g_new(SocketConnection, 1);
+    register gint result;
+    int flag = 1;
     if((connection->sock = socket(AF_INET, SOCK_STREAM, 0)) < 0){
         perror("opening stream socket");
         exit(1);
         }
     connection->host = g_strdup(host);
     connection->port = port;
+    result = setsockopt(connection->sock, IPPROTO_TCP, TCP_NODELAY,
+                                 (char*)&flag, sizeof(int));
+    if(result < 0)
+        perror("setting socket options");
     return connection;
     }
 
@@ -96,18 +103,36 @@ static void SocketConnection_destroy(SocketConnection *connection){
     }
 
 static gchar *SocketConnection_read(gint sock){
-    register gint len;
+    register gint i, len, line_complete = 0, line_expect = 1;
     register gchar *reply;
     register GString *string = g_string_sized_new(Socket_BUFSIZE);
+    register gboolean line_count_given = FALSE;
     gchar buffer[Socket_BUFSIZE+1];
     do {
-        if((len = recv(sock, buffer, Socket_BUFSIZE, 0)) < 0){
-            /* perror("reading from socket"); */
+        if((len = recv(sock, buffer, Socket_BUFSIZE, 0)) <= 0){
             len = 0;
+            break;
             }
         buffer[len] = '\0';
+        for(i = 0; i < len; i++)
+            if(buffer[i] == '\n')
+                line_complete++;
+        if((!string->len) && (len > 10))
+            if(!strncmp(buffer, "linecount:", 10)){
+                line_expect = atoi(&buffer[11]);
+                line_count_given = TRUE;
+                if(line_expect < 2)
+                    g_error("linecount: must be > 1");
+                }
+        if(line_complete > line_expect){
+            if(line_count_given)
+                g_error("Received [%d] socket message lines, but expected [%d]",
+                   line_complete, line_count_given);
+            else
+                g_error("Multiline socket messages must use linecount:");
+            }
         g_string_append(string, buffer);
-    } while(len == Socket_BUFSIZE);
+    } while(line_complete < line_expect);
     if(string->len){
         reply = string->str;
         g_string_free(string, FALSE);
@@ -117,12 +142,36 @@ static gchar *SocketConnection_read(gint sock){
     return NULL;
     }
 
+static void Socket_send_msg(gint sock, gchar *msg, gchar *err_msg){
+    register gint start = 0, len, msglen = strlen(msg);
+    do {
+        if((len = send(sock, msg+start, msglen-start, 0)) < 0){
+            perror(err_msg);
+            exit(1);
+            }
+        start += len;
+    } while(start < msglen);
+    return;
+    }
+
+static void Socket_send(gint sock, gchar *msg, gchar *err_msg){
+    register gint i, line_count = 0;
+    register GString *full_msg = g_string_sized_new(strlen(msg)+32);
+    for(i = 0; msg[i]; i++)
+        if(msg[i] == '\n')
+            line_count++;
+    if(line_count > 1)
+        g_string_sprintfa(full_msg, "linecount: %d\n", line_count+1);
+    g_string_sprintfa(full_msg, "%s", msg);
+    if(!line_count)
+        g_string_append_c(full_msg, '\n');
+    Socket_send_msg(sock, full_msg->str, err_msg);
+    g_string_free(full_msg, TRUE);
+    return;
+    }
+
 gchar *SocketClient_send(SocketClient *client, gchar *msg){
-    register gint len;
-    if((len = send(client->connection->sock, msg, strlen(msg), 0)) < 0){
-        perror("writing client message");
-        exit(1);
-        }
+    Socket_send(client->connection->sock, msg, "writing client message");
     return SocketConnection_read(client->connection->sock);
     }
 
@@ -144,6 +193,12 @@ static void SocketServer_reap_dead_children(int signum){
     return;
     }
 
+static void SocketServer_shutdown(int signum){
+    g_message("Server shutting down");
+    exit(0);
+    return;
+    }
+
 SocketServer *SocketServer_create(gint port, gint max_connections,
               SocketProcessFunc server_process_func,
               SocketConnectionOpenFunc connection_open_func,
@@ -174,17 +229,64 @@ SocketServer *SocketServer_create(gint port, gint max_connections,
         }
     server->connection->port = ntohs(sock_server.sin_port);
     signal(SIGCHLD, SocketServer_reap_dead_children);
+    signal(SIGTERM, SocketServer_shutdown);
+#ifdef USE_PTHREADS
+    server->sspd = g_new0(SocketServer_pthread_Data, max_connections);
+    pthread_mutex_init(&server->connection_mutex, NULL);
+#endif /* USE_PTHREADS */
     return server;
     }
 
-gboolean SocketServer_listen(SocketServer *server){
-    register int msgsock;
+static void SocketServer_process_connection(SocketServer *server, int msgsock){
+    register gpointer connection_data = server->connection_open_func
+                    ? server->connection_open_func(server->user_data)
+                    : NULL;
     register gchar *msg;
     register gboolean ok = TRUE;
-    register gpointer connection_data;
     gchar *reply;
+    do {
+        msg = SocketConnection_read(msgsock);
+        reply = NULL;
+        if(!msg)
+            break;
+        ok = server->server_process_func(msg, &reply, connection_data,
+                                         server->user_data);
+        g_free(msg);
+        if(reply){
+            Socket_send(msgsock, reply, "writing reply");
+            g_free(reply);
+            }
+    } while(ok);
+    if(server->connection_close_func)
+        server->connection_close_func(connection_data, server->user_data);
+    return;
+    }
+
+#ifdef USE_PTHREADS
+static void *SocketServer_pthread_func(void* data){
+    register SocketServer_pthread_Data *sspd = (SocketServer_pthread_Data*)data;
+    SocketServer_process_connection(sspd->server, sspd->msgsock);
+    pthread_mutex_lock(&sspd->server->connection_mutex);
+    g_message("cleaning up connection [%d]", global_connection_count);
+    global_connection_count--;
+    close(sspd->msgsock);
+    sspd->in_use = FALSE;
+    pthread_mutex_unlock(&sspd->server->connection_mutex);
+    pthread_exit(NULL);
+    return NULL;
+    }
+#endif /* USE_PTHREADS */
+
+gboolean SocketServer_listen(SocketServer *server){
+    register int msgsock;
+    register gint i;
     struct sockaddr_in client_addr;
     socklen_t client_len = sizeof(struct sockaddr_in);
+#ifdef USE_PTHREADS
+    pthread_attr_t pt_attr;
+    pthread_attr_init(&pt_attr);
+    pthread_attr_setdetachstate(&pt_attr, PTHREAD_CREATE_DETACHED);
+#endif /* USE_PTHREADS */
     /**/
     listen(server->connection->sock, 5);
     msgsock = accept(server->connection->sock,
@@ -195,32 +297,36 @@ gboolean SocketServer_listen(SocketServer *server){
         exit(1);
         }
     /* FIXME: send an busy warning message back to the client ? */
+#ifdef USE_PTHREADS
+    pthread_mutex_lock(&server->connection_mutex);
+    if(global_connection_count >= server->max_connections){
+        pthread_mutex_unlock(&server->connection_mutex);
+        g_message("Max connections reached");
+    } else {
+        for(i = 0; i < server->max_connections; i++)
+            if(!server->sspd[i].in_use){
+                server->sspd[i].in_use = TRUE;
+                break;
+                }
+        server->sspd[i].server = server;
+        server->sspd[i].msgsock = dup(msgsock);
+        if(server->sspd[i].msgsock < 0)
+            perror("duplicating socket for pthread");
+        global_connection_count++;
+        g_message("opened connection [%d/%d] from [%s]",
+                   global_connection_count,
+                   server->max_connections,
+                   inet_ntoa(client_addr.sin_addr));
+        pthread_mutex_unlock(&server->connection_mutex);
+        pthread_create(&server->sspd[i].thread, &pt_attr,
+                       SocketServer_pthread_func, (void*)&server->sspd[i]);
+        }
+#else /* USE_PTHREADS */
     if(global_connection_count >= server->max_connections){
         g_message("Max connections reached");
     } else {
         if(fork() == 0){
-            connection_data = server->connection_open_func
-                            ? server->connection_open_func(server->user_data)
-                            : NULL;
-            do {
-                msg = SocketConnection_read(msgsock);
-                reply = NULL;
-                if(!msg)
-                    break;
-                ok = server->server_process_func(msg, &reply, connection_data,
-                                                 server->user_data);
-                g_free(msg);
-                if(reply){
-                    if(send(msgsock, reply, strlen(reply), 0) < 0){
-                        perror("writing reply");
-                        exit(1);
-                        }
-                    g_free(reply);
-                    }
-            } while(ok);
-            if(server->connection_close_func)
-                server->connection_close_func(connection_data,
-                                              server->user_data);
+            SocketServer_process_connection(server, msgsock);
             exit(0);
         } else {
             global_connection_count++;
@@ -230,12 +336,17 @@ gboolean SocketServer_listen(SocketServer *server){
                     inet_ntoa(client_addr.sin_addr));
             }
         }
+#endif /* USE_PTHREADS */
     close(msgsock);
     return TRUE; /* Keep server running */
     }
 
 void SocketServer_destroy(SocketServer *server){
     SocketConnection_destroy(server->connection);
+#ifdef USE_PTHREADS
+    g_free(server->sspd);
+    pthread_mutex_destroy(&server->connection_mutex);
+#endif /* USE_PTHREADS */
     g_free(server);
     return;
     }
