@@ -22,6 +22,10 @@ extern "C" {
 
 #include <glib.h>
 
+#ifdef USE_PTHREADS
+#include <pthread.h>
+#endif /* USE_PTHREADS */
+
 typedef gboolean SocketProcessFunc(gchar *msg, gchar **reply,
                                    gpointer connection_data,
                                    gpointer user_data);
@@ -43,13 +47,26 @@ typedef struct {
     SocketConnection *connection;
 } SocketClient;
 
+#ifdef USE_PTHREADS
 typedef struct {
+               gboolean  in_use;
+    struct SocketServer *server;
+                    int  msgsock;
+              pthread_t  thread;
+} SocketServer_pthread_Data;
+#endif /* USE_PTHREADS */
+
+typedef struct SocketServer {
             SocketConnection *connection;
            SocketProcessFunc *server_process_func;
     SocketConnectionOpenFunc *connection_open_func;
    SocketConnectionCloseFunc *connection_close_func;
                     gpointer  user_data;
                         gint  max_connections;
+#ifdef USE_PTHREADS
+   SocketServer_pthread_Data *sspd;
+             pthread_mutex_t  connection_mutex;
+#endif /* USE_PTHREADS */
 } SocketServer;
 
 SocketClient *SocketClient_create(gchar *host, gint port);
