@@ -326,12 +326,20 @@ static SocketClient *Analysis_Client_connect(gchar *path){
     return NULL;
     }
 
-static gchar *AnalysisClient_send(SocketClient *sc, gchar *msg,
+static gchar *Analysis_Client_send(Analysis_Client *aclient, gchar *msg,
                                   gchar *expect, gboolean multi_line_reply){
-    register gchar *reply = SocketClient_send(sc, msg);
+    register gchar *reply = SocketClient_send(aclient->sc, msg);
     register gchar *line, *p = reply, *processed_reply;
     register gint line_count = 0;
     register GString *str = g_string_sized_new(64);
+    if(aclient->verbosity >= 3){
+        g_print("Message: client sent message [%s]\n", msg);
+        if((aclient->verbosity == 3) && (strlen(reply) >= 80))
+            g_print("Message: client received reply [%.*s<truncated>] \n",
+                               80, reply);
+        else
+            g_print("Message: client received reply [%s]\n", reply);
+        }
     do {
         while(*p){
             if(!isspace(*p)) /* Strip blank lines */
@@ -343,6 +351,12 @@ static gchar *AnalysisClient_send(SocketClient *sc, gchar *msg,
             g_error("Error from server: [%s]", p);
         if(!strncmp(p, "warning:", 8)){
             g_warning("Warning from server: [%s]", p);
+        } else if(!strncmp(p, "linecount:", 8)){
+            while(*p){ /* Skip to end of line */
+                if(*p == '\n')
+                    break;
+                p++;
+                }
         } else if(!strncmp(p, expect, strlen(expect))){
             while(*p){ /* Skip to end of line */
                 if(*p == '\n')
@@ -370,55 +384,88 @@ static gchar *AnalysisClient_send(SocketClient *sc, gchar *msg,
     return processed_reply;
     }
 
-static void Analysis_Client_set_param(SocketClient *sc){
+static void Analysis_Client_set_param(Analysis_Client *aclient, GAM *gam){
     register HSPset_ArgumentSet *has = HSPset_ArgumentSet_create(NULL);
     register Analysis_ArgumentSet *aas = Analysis_ArgumentSet_create(NULL);
     register gchar *msg, *reply;
     /**/
     if(aas->custom_server_command){
         msg = g_strdup_printf("%s\n", aas->custom_server_command);
-        reply = AnalysisClient_send(sc, msg, "ok:", FALSE);
+        reply = Analysis_Client_send(aclient, msg, "ok:", FALSE);
         g_free(msg);
         g_free(reply);
         }
     /**/
-    msg = g_strdup_printf("set param seedrepeat %d", has->seed_repeat);
-    reply = AnalysisClient_send(sc, msg, "ok:", FALSE);
+    msg = g_strdup_printf("set param seedrepeat %d",
+                          has->seed_repeat);
+    reply = Analysis_Client_send(aclient, msg, "ok:", FALSE);
     g_free(msg);
     g_free(reply);
     /**/
-    msg = g_strdup_printf("set param dnahspthreshold %d", has->dna_hsp_threshold);
-    reply = AnalysisClient_send(sc, msg, "ok:", FALSE);
+    msg = g_strdup_printf("set param dnahspthreshold %d",
+                          has->dna_hsp_threshold);
+    reply = Analysis_Client_send(aclient, msg, "ok:", FALSE);
     g_free(msg);
     g_free(reply);
     /**/
     msg = g_strdup_printf("set param proteinhspthreshold %d",
                           has->protein_hsp_threshold);
-    reply = AnalysisClient_send(sc, msg, "ok:", FALSE);
+    reply = Analysis_Client_send(aclient, msg, "ok:", FALSE);
     g_free(msg);
     g_free(reply);
     /**/
     msg = g_strdup_printf("set param codonhspthreshold %d",
                           has->codon_hsp_threshold);
-    reply = AnalysisClient_send(sc, msg, "ok:", FALSE);
+    reply = Analysis_Client_send(aclient, msg, "ok:", FALSE);
+    g_free(msg);
+    g_free(reply);
+    /**/
+    msg = g_strdup_printf("set param dnahspdropoff %d",
+                          has->dna_hsp_dropoff);
+    reply = Analysis_Client_send(aclient, msg, "ok:", FALSE);
+    g_free(msg);
+    g_free(reply);
+    /**/
+    msg = g_strdup_printf("set param proteinhspdropoff %d",
+                          has->protein_hsp_dropoff);
+    reply = Analysis_Client_send(aclient, msg, "ok:", FALSE);
+    g_free(msg);
+    g_free(reply);
+    /**/
+    msg = g_strdup_printf("set param codonhspdropoff %d",
+                          has->codon_hsp_dropoff);
+    reply = Analysis_Client_send(aclient, msg, "ok:", FALSE);
     g_free(msg);
     g_free(reply);
     /**/
     msg = g_strdup_printf("set param geneseedthreshold %d",
                           has->geneseed_threshold);
-    reply = AnalysisClient_send(sc, msg, "ok:", FALSE);
+    reply = Analysis_Client_send(aclient, msg, "ok:", FALSE);
     g_free(msg);
     g_free(reply);
     /**/
     msg = g_strdup_printf("set param geneseedrepeat %d",
                           has->geneseed_repeat);
-    reply = AnalysisClient_send(sc, msg, "ok:", FALSE);
+    reply = Analysis_Client_send(aclient, msg, "ok:", FALSE);
     g_free(msg);
     g_free(reply);
+    /**/
+    msg = g_strdup_printf("set param maxqueryspan %d",
+                          gam->max_query_span);
+    reply = Analysis_Client_send(aclient, msg, "ok:", FALSE);
+    g_free(msg);
+    g_free(reply);
+    /**/
+    msg = g_strdup_printf("set param maxtargetspan %d",
+                          gam->max_target_span);
+    reply = Analysis_Client_send(aclient, msg, "ok:", FALSE);
+    g_free(msg);
+    g_free(reply);
+    /**/
     return;
     }
 
-static Analysis_Client *Analysis_Client_create(gchar *path){
+static Analysis_Client *Analysis_Client_create(gchar *path, gint verbosity){
     register Analysis_Client *aclient;
     register SocketClient *sc = Analysis_Client_connect(path);
     register Alphabet_Type alphabet_type;
@@ -427,8 +474,9 @@ static Analysis_Client *Analysis_Client_create(gchar *path){
         return NULL;
     aclient = g_new(Analysis_Client, 1);
     aclient->sc = sc;
+    aclient->verbosity = verbosity;
     aclient->probe_fdb = NULL;
-    dbinfo = AnalysisClient_send(sc, "dbinfo", "dbinfo:", FALSE);
+    dbinfo = Analysis_Client_send(aclient, "dbinfo", "dbinfo:", FALSE);
     dbinfo_word = g_strsplit(dbinfo, " ", 8);
     /**/
     g_assert(dbinfo_word[0]);
@@ -444,8 +492,12 @@ static Analysis_Client *Analysis_Client_create(gchar *path){
         alphabet_type = Alphabet_Type_PROTEIN;
     /**/
     aclient->is_masked = FALSE;
-    if(!strcmp(dbinfo_word[2], "masked"))
+    if(!strcmp(dbinfo_word[2], "softmasked"))
         aclient->is_masked = TRUE;
+    else if(!strcmp(dbinfo_word[2], "unmasked"))
+        aclient->is_masked = TRUE;
+    else
+        g_error("Unrecognised dbinfo masking state [%s]", dbinfo_word[2]);
     /**/
     aclient->server_alphabet = Alphabet_create(alphabet_type, aclient->is_masked);
     aclient->num_seqs      = atoll(dbinfo_word[3]);
@@ -456,7 +508,6 @@ static Analysis_Client *Analysis_Client_create(gchar *path){
     g_free(dbinfo);
     aclient->curr_query = NULL;
     aclient->seq_cache = g_new0(Sequence*, aclient->num_seqs);
-    Analysis_Client_set_param(sc);
     return aclient;
     }
 
@@ -489,7 +540,7 @@ static void Analysis_Client_set_probe_fdb(Analysis_Client *aclient,
 static void Analysis_Client_set_query(Analysis_Client *aclient, Sequence *seq){
     register gchar *seq_str = Sequence_get_str(seq);
     register gchar *msg = g_strdup_printf("set query %s", seq_str);
-    register gchar *reply = AnalysisClient_send(aclient->sc, msg, "ok:", FALSE);
+    register gchar *reply = Analysis_Client_send(aclient, msg, "ok:", FALSE);
     register gchar **word;
     register gint len, checksum;
     if(strncmp(reply, "ok:", 3))
@@ -513,7 +564,7 @@ static void Analysis_Client_set_query(Analysis_Client *aclient, Sequence *seq){
     }
 
 static void Analysis_Client_revcomp_query(Analysis_Client *aclient){
-    register gchar *reply = AnalysisClient_send(aclient->sc,
+    register gchar *reply = Analysis_Client_send(aclient,
             "revcomp query", "ok: query strand revcomp", FALSE);
     register Sequence *curr_query;
     curr_query = aclient->curr_query;
@@ -524,7 +575,7 @@ static void Analysis_Client_revcomp_query(Analysis_Client *aclient){
     }
 
 static void Analysis_Client_revcomp_target(Analysis_Client *aclient){
-    register gchar *reply = AnalysisClient_send(aclient->sc,
+    register gchar *reply = Analysis_Client_send(aclient,
             "revcomp target", "ok: target strand", FALSE);
     g_free(reply);
     return;
@@ -566,12 +617,13 @@ static SparseCache_Page *Analysis_Client_SparseCache_fill_func(gint start,
                   page_len;
     register gchar *msg = g_strdup_printf("get subseq %d %d %d",
             key->target_id, start, len);
-    register gchar *reply = AnalysisClient_send(key->aclient->sc, msg,
+    register gchar *reply = Analysis_Client_send(key->aclient, msg,
                                                 "subseq:", FALSE);
     if(strncmp(reply, "subseq:", 7))
         g_error("Failed to get subseq for target (%d,%d,%d) [%s]",
                 key->target_id, start, len, reply);
     page->get_func = Analysis_Client_SparseCache_get_func;
+    page->copy_func = NULL;
     page_len = strlen(reply+8)-1;
     page->data = g_strndup(reply+8, page_len);
     page->data_size = sizeof(gchar)*page_len;
@@ -588,12 +640,13 @@ static void Analysis_Client_SparseCache_free_func(gpointer user_data){
     return;
     }
 
-static SparseCache *Analysis_Client_get_SparseCache(Analysis_Client *aclient,
-                                                    gint sequence_id, gint len){
+static SparseCache *Analysis_Client_get_SparseCache(
+                    Analysis_Client *aclient,
+                    gint sequence_id, gint len){
     register Analysis_Client_Key *key
         = Analysis_Client_Key_create(aclient, sequence_id, len);
-    return SparseCache_create(len, Analysis_Client_SparseCache_fill_func, NULL,
-                              Analysis_Client_SparseCache_free_func, key);
+    return SparseCache_create(len, Analysis_Client_SparseCache_fill_func,
+                         NULL, Analysis_Client_SparseCache_free_func, key);
     }
 
 static Sequence *Analysis_Client_get_Sequence(Analysis_Client *aclient,
@@ -610,7 +663,7 @@ static Sequence *Analysis_Client_get_Sequence(Analysis_Client *aclient,
         return Sequence_share(seq);
         }
     msg = g_strdup_printf("get info %d", sequence_id);
-    reply = AnalysisClient_send(aclient->sc, msg, "seqinfo:", FALSE);
+    reply = Analysis_Client_send(aclient, msg, "seqinfo:", FALSE);
     /**/
     if(strncmp(reply, "seqinfo:", 8))
         g_error("Failed to set info for target [%d]", sequence_id);
@@ -690,7 +743,7 @@ static void Analysis_Client_get_hsp_sets(Analysis_Client *aclient,
                                          Analysis *analysis,
                                          gboolean swap_chains,
                                          gboolean revcomp_target){
-    register gchar *reply = AnalysisClient_send(aclient->sc,
+    register gchar *reply = Analysis_Client_send(aclient,
                                                 "get hsps", "hspset:", TRUE);
     gint pos = 0, intval = 0;
     register gboolean ok = TRUE;
@@ -851,14 +904,17 @@ Analysis *Analysis_create(
     g_assert(query_path_list->len);
     g_assert(target_path_list->len);
     analysis->aas = Analysis_ArgumentSet_create(NULL);
+
     analysis->verbosity = verbosity;
     /**/
     if(query_path_list->len == 1)
         analysis->query_ac
-                = Analysis_Client_create((gchar*)query_path_list->pdata[0]);
+                = Analysis_Client_create((gchar*)query_path_list->pdata[0],
+                                         verbosity);
     if(target_path_list->len == 1)
         analysis->target_ac
-                = Analysis_Client_create((gchar*)target_path_list->pdata[0]);
+                = Analysis_Client_create((gchar*)target_path_list->pdata[0],
+                                       verbosity);
     /**/
     if(query_type == Alphabet_Type_UNKNOWN){
         if(analysis->query_ac){
@@ -896,6 +952,12 @@ Analysis *Analysis_create(
                                mas->translate,
                                analysis->aas->use_exhaustive,
                                verbosity);
+    /**/
+    if(analysis->query_ac)
+        Analysis_Client_set_param(analysis->query_ac, analysis->gam);
+    if(analysis->target_ac)
+        Analysis_Client_set_param(analysis->target_ac, analysis->gam);
+    /**/
     Analysis_find_matches(analysis, &dna_match, &protein_match,
                                     &codon_match);
     match = dna_match;
