@@ -413,6 +413,7 @@ static GPtrArray *GAM_build_match_list(C4_Model *model){
             }
         }
     g_assert(match_list->len);
+    g_ptr_array_free(match_transition_list, TRUE);
     return match_list;
     }
 
@@ -421,6 +422,8 @@ GAM *GAM_create(Alphabet_Type query_type, Alphabet_Type target_type,
                 Translate *translate, gboolean use_exhaustive,
                 gint verbosity){
     register GAM *gam = g_new0(GAM, 1);
+    register gint i;
+    register C4_Span *span;
     gam->ref_count = 1;
     gam->dna_submat = Submat_share(dna_submat);
     gam->protein_submat = Submat_share(protein_submat);
@@ -468,6 +471,15 @@ GAM *GAM_create(Alphabet_Type query_type, Alphabet_Type target_type,
             }
         }
     gam->verbosity = verbosity;
+    /* Find max_{query,target}_span */
+    gam->max_query_span = gam->max_target_span = 0;
+    for(i = 0; i < gam->model->span_list->len; i++){
+        span = gam->model->span_list->pdata[i];
+        if(gam->max_query_span < span->max_query)
+            gam->max_query_span = span->max_query;
+        if(gam->max_target_span < span->max_target)
+            gam->max_target_span = span->max_target;
+        }
     return gam;
     }
 
@@ -679,6 +691,8 @@ static Alignment *GAM_Result_refine_alignment(GAM_Result *gam_result,
         }
     g_assert(refined_alignment);
     g_assert(refined_alignment->score >= alignment->score);
+    if(gam_result->gam->verbosity > 1)
+        g_message("Refined alignment score [%d]", refined_alignment->score);
     return refined_alignment;
     }
 
@@ -688,7 +702,6 @@ static void GAM_Result_add_alignment(GAM_Result *gam_result,
     register Alignment *refined_alignment;
     if(!gam_result->alignment_list)
         gam_result->alignment_list = g_ptr_array_new();
-    g_ptr_array_add(gam_result->alignment_list, alignment);
     if(gam_result->gam->gas->refinement != GAM_Refinement_NONE){
         refined_alignment = GAM_Result_refine_alignment(gam_result, alignment);
         Alignment_destroy(alignment);
@@ -699,6 +712,7 @@ static void GAM_Result_add_alignment(GAM_Result *gam_result,
             return;
             }
         }
+    g_ptr_array_add(gam_result->alignment_list, alignment);
     SubOpt_add_alignment(gam_result->subopt, alignment);
     return;
     }
@@ -920,8 +934,7 @@ typedef struct {
     GPtrArray *hsp_list;
     RangeTree *rangetree;
           HSP *max_cobs_hsp;
-         gint  max_query_span;
-         gint  max_target_span;
+          GAM *gam;
 } GAM_Geneseed_Data;
 
 static void GAM_Result_geneseed_add(HSPset *hspset, GAM_Geneseed_Data *gsd){
@@ -972,11 +985,11 @@ static void GAM_Result_geneseed_visit_hsp_fwd(GAM_Geneseed_Data *gsd,
     if(!gsd->fwd_keep[hsp_id]){
         gsd->fwd_keep[hsp_id] = TRUE;
         hsp = gsd->hsp_list->pdata[hsp_id];
-        query_range = gsd->max_query_span
+        query_range = gsd->gam->max_query_span
                     + (((HSP_query_end(hsp) - HSP_query_cobs(hsp))
                        + (HSP_query_cobs(gsd->max_cobs_hsp)
                          - gsd->max_cobs_hsp->query_start)) * 2);
-        target_range = gsd->max_target_span
+        target_range = gsd->gam->max_target_span
                      + (((HSP_target_end(hsp) - HSP_target_cobs(hsp))
                        + (HSP_target_cobs(gsd->max_cobs_hsp)
                          - gsd->max_cobs_hsp->target_start)) * 2);
@@ -984,11 +997,11 @@ static void GAM_Result_geneseed_visit_hsp_fwd(GAM_Geneseed_Data *gsd,
         /*
         g_message("find fwd (%d,%d,%d) (%d,%d,%d) [%d,%d]",
                     (HSP_query_end(hsp) - HSP_query_cobs(hsp)),
-                    gsd->max_query_span,
+                    gsd->gam->max_query_span,
                     (HSP_query_cobs(gsd->max_cobs_hsp)
                       - gsd->max_cobs_hsp->query_start),
                     (HSP_target_end(hsp) - HSP_target_cobs(hsp)),
-                    gsd->max_target_span,
+                    gsd->gam->max_target_span,
                     (HSP_target_cobs(gsd->max_cobs_hsp)
                       - gsd->max_cobs_hsp->target_start),
                     query_range, target_range);
@@ -1010,22 +1023,22 @@ static void GAM_Result_geneseed_visit_hsp_rev(GAM_Geneseed_Data *gsd,
     if(!gsd->rev_keep[hsp_id]){
         gsd->rev_keep[hsp_id] = TRUE;
         hsp = gsd->hsp_list->pdata[hsp_id];
-        query_range = gsd->max_query_span
+        query_range = gsd->gam->max_query_span
                     + (((HSP_query_end(hsp) - HSP_query_cobs(hsp))
                        + (HSP_query_cobs(gsd->max_cobs_hsp)
                          - gsd->max_cobs_hsp->query_start)) * 2);
-        target_range = gsd->max_target_span
+        target_range = gsd->gam->max_target_span
                      + (((HSP_target_end(hsp) - HSP_target_cobs(hsp))
                        + (HSP_target_cobs(gsd->max_cobs_hsp)
                          - gsd->max_cobs_hsp->target_start)) * 2);
         /*
         g_message("find rev (%d,%d,%d) (%d,%d,%d) [%d,%d]",
                     (HSP_query_end(hsp) - HSP_query_cobs(hsp)),
-                    gsd->max_query_span,
+                    gsd->gam->max_query_span,
                     (HSP_query_cobs(gsd->max_cobs_hsp)
                       - gsd->max_cobs_hsp->query_start),
                     (HSP_target_end(hsp) - HSP_target_cobs(hsp)),
-                    gsd->max_target_span,
+                    gsd->gam->max_target_span,
                     (HSP_target_cobs(gsd->max_cobs_hsp)
                       - gsd->max_cobs_hsp->target_start),
                     query_range, target_range);
@@ -1071,13 +1084,11 @@ static gint GAM_Result_geneseed_select(HSPset *hspset,
 static void GAM_Result_geneseed_filter(GAM *gam, Comparison *comparison){
     register gint i, hsp_id = 0;
     register HSP *hsp;
-    register C4_Span *span;
     GAM_Geneseed_Data gsd;
     gsd.hsp_list = g_ptr_array_new();
     gsd.rangetree = RangeTree_create();
     gsd.max_cobs_hsp = NULL;
-    gsd.max_query_span = 0;
-    gsd.max_target_span = 0;
+    gsd.gam = gam;
     /* Build the rangetree */
     GAM_Result_geneseed_add(comparison->dna_hspset, &gsd);
     GAM_Result_geneseed_add(comparison->protein_hspset, &gsd);
@@ -1086,13 +1097,6 @@ static void GAM_Result_geneseed_filter(GAM *gam, Comparison *comparison){
     g_assert(gsd.max_cobs_hsp);
     gsd.fwd_keep = g_new0(gboolean, gsd.hsp_list->len);
     gsd.rev_keep = g_new0(gboolean, gsd.hsp_list->len);
-    for(i = 0; i < gam->model->span_list->len; i++){
-        span = gam->model->span_list->pdata[i];
-        if(gsd.max_query_span < span->max_query)
-            gsd.max_query_span = span->max_query;
-        if(gsd.max_target_span < span->max_target)
-            gsd.max_target_span = span->max_target;
-        }
     /* Find HSPs reachable from geneseed HSPs */
     for(i = 0; i < gsd.hsp_list->len; i++){
         hsp = gsd.hsp_list->pdata[i];
@@ -1104,8 +1108,7 @@ static void GAM_Result_geneseed_filter(GAM *gam, Comparison *comparison){
     /* Keep HSPs marked as keep */
     hsp_id = GAM_Result_geneseed_select(comparison->dna_hspset,
                  gsd.fwd_keep, gsd.rev_keep, hsp_id);
-    hsp_id = GAM_Result_geneseed_select(comparison->protein_hspset,
-                 gsd.fwd_keep, gsd.rev_keep, hsp_id);
+    hsp_id = GAM_Result_geneseed_select(comparison->protein_hspset, gsd.fwd_keep, gsd.rev_keep, hsp_id);
     hsp_id = GAM_Result_geneseed_select(comparison->codon_hspset,
                  gsd.fwd_keep, gsd.rev_keep, hsp_id);
     /**/
@@ -1125,13 +1128,14 @@ static void GAM_Result_geneseed_filter(GAM *gam, Comparison *comparison){
         comparison->codon_hspset = NULL;
         }
     /**/
+    /**/
     /*
     g_message("start with [%d] FILTER down to [%d]",
             gsd.hsp_list->len,
         (comparison->dna_hspset?comparison->dna_hspset->hsp_list->len:0)
        +(comparison->protein_hspset?comparison->protein_hspset->hsp_list->len:0)
        +(comparison->codon_hspset?comparison->codon_hspset->hsp_list->len:0));
-    */
+       */
     RangeTree_destroy(gsd.rangetree, NULL, NULL);
     g_free(gsd.fwd_keep);
     g_free(gsd.rev_keep);
@@ -1144,8 +1148,14 @@ GAM_Result *GAM_Result_heuristic_create(GAM *gam,
     register GAM_Result *gam_result;
     register HSPset_ArgumentSet *has
         = Comparison_Param_get_HSPSet_Argument_Set(comparison->param);
-    if(has->geneseed_threshold)
+    if(has->geneseed_threshold){
+        /* Raise score threshold to geneseed
+         * to prevent low-scoring subopt alignments.
+         */
+        if(gam->gas->threshold < has->geneseed_threshold)
+            gam->gas->threshold = has->geneseed_threshold;
         GAM_Result_geneseed_filter(gam, comparison);
+        }
     if(!Comparison_has_hsps(comparison))
         return NULL;
     /*
