@@ -104,6 +104,8 @@ typedef struct {
                  GTree *percent_threshold_tree;
                         /* Contains GAM_QueryInfo */
              PQueueSet *pqueue_set;
+                  gint  max_query_span;
+                  gint  max_target_span;
 } GAM;
 
 GAM *GAM_create(Alphabet_Type query_type, Alphabet_Type target_type,
