@@ -32,7 +32,7 @@ typedef struct {
 } Exonerate_Server;
 
 static void Exonerate_Server_memory_usage(Exonerate_Server *exonerate_server){
-    register gsize dataset_memory, index_memory;
+    register guint64 dataset_memory, index_memory;
     dataset_memory = Dataset_memory_usage(exonerate_server->dataset),
     index_memory = exonerate_server->index
                  ? (Index_memory_usage(exonerate_server->index) - dataset_memory)
@@ -41,7 +41,7 @@ static void Exonerate_Server_memory_usage(Exonerate_Server *exonerate_server){
         g_message("Memory usage: dataset: %d Mb, index: %d Mb, Total %d Mb",
               (gint)(dataset_memory >> 20),
               (gint)(index_memory >> 20),
-              (gint)(dataset_memory+index_memory) >> 20);
+              (gint)((dataset_memory+index_memory) >> 20));
     return;
     }
 
@@ -93,8 +93,13 @@ typedef struct {
                gint  dna_hsp_threshold;
                gint  protein_hsp_threshold;
                gint  codon_hsp_threshold;
+               gint  dna_hsp_dropoff;
+               gint  protein_hsp_dropoff;
+               gint  codon_hsp_dropoff;
                gint  geneseed_threshold;
                gint  geneseed_repeat;
+               gint  max_query_span;
+               gint  max_target_span;
 } Exonerate_Server_Connection;
 
 static Exonerate_Server_Connection *Exonerate_Server_Connection_create(void){
@@ -113,8 +118,13 @@ static Exonerate_Server_Connection *Exonerate_Server_Connection_create(void){
     connection->dna_hsp_threshold = has->dna_hsp_threshold;
     connection->protein_hsp_threshold = has->protein_hsp_threshold;
     connection->codon_hsp_threshold = has->codon_hsp_threshold;
+    connection->dna_hsp_dropoff = has->dna_hsp_dropoff;
+    connection->protein_hsp_dropoff = has->protein_hsp_dropoff;
+    connection->codon_hsp_dropoff = has->codon_hsp_dropoff;
     connection->geneseed_threshold = has->geneseed_threshold;
     connection->geneseed_repeat = has->geneseed_repeat;
+    connection->max_query_span = 0;
+    connection->max_target_span = 0;
     return connection;
     }
 
@@ -133,6 +143,7 @@ static void Exonerate_Server_Connection_destroy(
 /**/
 
 static gpointer Exonerate_Server_Connection_open(gpointer user_data){
+    g_message("[%s]", __FUNCTION__);
     return Exonerate_Server_Connection_create();
     }
 
@@ -140,6 +151,7 @@ static void Exonerate_Server_Connection_close(gpointer connection_data,
                                               gpointer user_data){
     register Exonerate_Server_Connection *server_connection
         = connection_data;
+    g_message("[%s]", __FUNCTION__);
     Exonerate_Server_Connection_destroy(server_connection);
     return;
     }
@@ -212,8 +224,11 @@ static gchar *Exonerate_Server_help(void){
         "        dnahspthreshold\n"
         "        proteinhspthreshold\n"
         "        codonhspthreshold\n"
+        "\n"
         "        geneseedthreshold\n"
         "        geneseedrepeat\n"
+        "        maxqueryspan\n"
+        "        maxtargetspan\n"
         "--\n");
     }
 
@@ -296,21 +311,17 @@ static gchar *Exonerate_Server_get_hsps(Exonerate_Server *exonerate_server,
         return g_strdup_printf(
                 "error: revcomp target only available for protein2dna matches");
     if(connection->geneseed_threshold > 0){
-        /* FIXME: temp disable of server-side geneseed */
-        /* g_warning("server-side geneseed not yet implemeted"); */
-        /**/
-        index_hsp_set_list = Index_get_HSPsets(exonerate_server->index,
-                                               connection->hsp_param,
-                                               connection->query,
-                                               connection->revcomp_target);
-        /*
+        if(connection->geneseed_threshold < connection->hsp_param->threshold)
+            return g_strdup_printf(
+                    "error: geneseed threshold must be >= hsp threshold");
         index_hsp_set_list = Index_get_HSPsets_geneseed(exonerate_server->index,
                                                connection->hsp_param,
                                                connection->query,
                                                connection->revcomp_target,
                                                connection->geneseed_threshold,
-                                               connection->geneseed_repeat);
-        */
+                                               connection->geneseed_repeat,
+                                               connection->max_query_span,
+                                               connection->max_target_span);
     } else {
         index_hsp_set_list = Index_get_HSPsets(exonerate_server->index,
                                                connection->hsp_param,
@@ -356,7 +367,7 @@ static Sequence *Exonerate_Server_get_query(Index *index,
         if(alphabet_type == Alphabet_Type_DNA){
             connection->query_strand = Sequence_Strand_FORWARD;
         } else {
-            g_assert(alphabet_type = Alphabet_Type_PROTEIN);
+            g_assert(alphabet_type == Alphabet_Type_PROTEIN);
             connection->query_strand = Sequence_Strand_UNKNOWN;
             if((index->dataset->alphabet->type == Alphabet_Type_DNA)
             && (!(index->header->type & 1))){
@@ -373,12 +384,21 @@ static Sequence *Exonerate_Server_get_query(Index *index,
         g_assert(match);
         connection->hsp_param = HSP_Param_create(match, FALSE);
         connection->hsp_param->seed_repeat = connection->seed_repeat;
+        /**/
         HSP_Param_set_dna_hsp_threshold(connection->hsp_param,
                                         connection->dna_hsp_threshold);
         HSP_Param_set_protein_hsp_threshold(connection->hsp_param,
                                             connection->protein_hsp_threshold);
         HSP_Param_set_codon_hsp_threshold(connection->hsp_param,
                                           connection->codon_hsp_threshold);
+        /**/
+        HSP_Param_set_dna_hsp_dropoff(connection->hsp_param,
+                                      connection->dna_hsp_dropoff);
+        HSP_Param_set_protein_hsp_dropoff(connection->hsp_param,
+                                          connection->protein_hsp_dropoff);
+        HSP_Param_set_codon_hsp_dropoff(connection->hsp_param,
+                                        connection->codon_hsp_dropoff);
+        /**/
         g_assert(connection->hsp_param);
         HSP_Param_set_wordlen(connection->hsp_param, index->header->word_length);
         }
@@ -396,9 +416,11 @@ static gchar *Exonerate_Server_set_param_seedrepeat(
     connection->seed_repeat = seed_repeat;
     if(connection->hsp_param)
         connection->hsp_param->seed_repeat = seed_repeat;
-    return g_strdup_printf("ok:\n");
+    return g_strdup_printf("ok: set\n");
     }
 
+/**/
+
 static gchar *Exonerate_Server_set_param_dnahspthreshold(
               Exonerate_Server_Connection *connection, GPtrArray *word_list){
     register gint dnahspthreshold = atoi(word_list->pdata[3]);
@@ -407,7 +429,7 @@ static gchar *Exonerate_Server_set_param_dnahspthreshold(
     connection->dna_hsp_threshold = dnahspthreshold;
     if(connection->hsp_param)
         HSP_Param_set_dna_hsp_threshold(connection->hsp_param, dnahspthreshold);
-    return g_strdup_printf("ok:\n");
+    return g_strdup_printf("ok: set\n");
     }
 
 static gchar *Exonerate_Server_set_param_proteinhspthreshold(
@@ -419,7 +441,7 @@ static gchar *Exonerate_Server_set_param_proteinhspthreshold(
     if(connection->hsp_param)
         HSP_Param_set_protein_hsp_threshold(connection->hsp_param,
                                             proteinhspthreshold);
-    return g_strdup_printf("ok:\n");
+    return g_strdup_printf("ok: set\n");
     }
 
 static gchar *Exonerate_Server_set_param_codonhspthreshold(
@@ -431,16 +453,55 @@ static gchar *Exonerate_Server_set_param_codonhspthreshold(
     if(connection->hsp_param)
         HSP_Param_set_codon_hsp_threshold(connection->hsp_param,
                                           codonhspthreshold);
-    return g_strdup_printf("ok:\n");
+    return g_strdup_printf("ok: set\n");
     }
 
+/**/
+
+static gchar *Exonerate_Server_set_param_dnahspdropoff(
+              Exonerate_Server_Connection *connection, GPtrArray *word_list){
+    register gint dnahspdropoff = atoi(word_list->pdata[3]);
+    if(dnahspdropoff < 0)
+        return g_strdup_printf("error: dnahspdropoff must be >= 0\n");
+    connection->dna_hsp_dropoff = dnahspdropoff;
+    if(connection->hsp_param)
+        HSP_Param_set_dna_hsp_dropoff(connection->hsp_param, dnahspdropoff);
+    return g_strdup_printf("ok: set\n");
+    }
+
+static gchar *Exonerate_Server_set_param_proteinhspdropoff(
+              Exonerate_Server_Connection *connection, GPtrArray *word_list){
+    register gint proteinhspdropoff = atoi(word_list->pdata[3]);
+    if(proteinhspdropoff < 0)
+        return g_strdup_printf("error: proteinhspdropoff must be >= 0\n");
+    connection->protein_hsp_dropoff = proteinhspdropoff;
+    if(connection->hsp_param)
+        HSP_Param_set_protein_hsp_dropoff(connection->hsp_param,
+                                          proteinhspdropoff);
+    return g_strdup_printf("ok: set\n");
+    }
+
+static gchar *Exonerate_Server_set_param_codonhspdropoff(
+              Exonerate_Server_Connection *connection, GPtrArray *word_list){
+    register gint codonhspdropoff = atoi(word_list->pdata[3]);
+    if(codonhspdropoff < 0)
+        return g_strdup_printf("error: codonhspdropoff must be >= 0\n");
+    connection->codon_hsp_dropoff = codonhspdropoff;
+    if(connection->hsp_param)
+        HSP_Param_set_codon_hsp_dropoff(connection->hsp_param,
+                                        codonhspdropoff);
+    return g_strdup_printf("ok: set\n");
+    }
+
+/**/
+
 static gchar *Exonerate_Server_set_param_geneseedthreshold(
               Exonerate_Server_Connection *connection, GPtrArray *word_list){
     register gint geneseed_threshold = atoi(word_list->pdata[3]);
     if(geneseed_threshold < 0)
         return g_strdup_printf("error: geneseed_threshold must be >= 0\n");
     connection->geneseed_threshold = geneseed_threshold;
-    return g_strdup_printf("ok:\n");
+    return g_strdup_printf("ok: set\n");
     }
 
 static gchar *Exonerate_Server_set_param_geneseedrepeat(
@@ -449,7 +510,25 @@ static gchar *Exonerate_Server_set_param_geneseedrepeat(
     if(geneseed_repeat <= 1)
         return g_strdup_printf("error: geneseed_repeat must be > 1\n");
     connection->geneseed_repeat = geneseed_repeat;
-    return g_strdup_printf("ok:\n");
+    return g_strdup_printf("ok: set\n");
+    }
+
+static gchar *Exonerate_Server_set_param_max_query_span(
+              Exonerate_Server_Connection *connection, GPtrArray *word_list){
+    register gint max_query_span= atoi(word_list->pdata[3]);
+    if(max_query_span < 0)
+        return g_strdup_printf("error: max_query_span must be >= 0\n");
+    connection->max_query_span = max_query_span;
+    return g_strdup_printf("ok: set\n");
+    }
+
+static gchar *Exonerate_Server_set_param_max_target_span(
+              Exonerate_Server_Connection *connection, GPtrArray *word_list){
+    register gint max_target_span= atoi(word_list->pdata[3]);
+    if(max_target_span < 0)
+        return g_strdup_printf("error: max_target_span must be >= 0\n");
+    connection->max_target_span = max_target_span;
+    return g_strdup_printf("ok: set\n");
     }
 
 static gchar *Exonerate_Server_set_param(Exonerate_Server_Connection *connection,
@@ -466,12 +545,26 @@ static gchar *Exonerate_Server_set_param(Exonerate_Server_Connection *connection
     } else if(!strcmp(name, "codonhspthreshold")){
         reply = Exonerate_Server_set_param_codonhspthreshold(connection,
                                                              word_list);
+    } else if(!strcmp(name, "dnahspdropoff")){
+        reply = Exonerate_Server_set_param_dnahspdropoff(connection, word_list);
+    } else if(!strcmp(name, "proteinhspdropoff")){
+        reply = Exonerate_Server_set_param_proteinhspdropoff(connection,
+                                                             word_list);
+    } else if(!strcmp(name, "codonhspdropoff")){
+        reply = Exonerate_Server_set_param_codonhspdropoff(connection,
+                                                           word_list);
     } else if(!strcmp(name, "geneseedthreshold")){
         reply = Exonerate_Server_set_param_geneseedthreshold(connection,
                                                              word_list);
     } else if(!strcmp(name, "geneseedrepeat")){
         reply = Exonerate_Server_set_param_geneseedrepeat(connection,
                                                           word_list);
+    } else if(!strcmp(name, "maxqueryspan")){
+        reply = Exonerate_Server_set_param_max_query_span(connection,
+                                                          word_list);
+    } else if(!strcmp(name, "maxtargetspan")){
+        reply = Exonerate_Server_set_param_max_target_span(connection,
+                                                           word_list);
     } else {
         reply = g_strdup_printf(
                 "warning: set param %s ignored by server\n", name);
@@ -491,6 +584,8 @@ static gboolean Exonerate_Server_process(gchar *msg, gchar **reply,
     register gint start, len, num;
     register Exonerate_Server_Connection *connection = connection_data;
     g_assert(msg);
+    if(server->verbosity >= 3)
+        g_print("Message: server received command [%s]\n", msg);
     if(!msg[msg_len-1] == '\n')
         msg[--msg_len] = '\0';
     word_list = Exonerate_Server_get_word_list(msg);
@@ -505,13 +600,13 @@ static gboolean Exonerate_Server_process(gchar *msg, gchar **reply,
             (*reply) = NULL;
             keep_connection = FALSE;
         } else if(!strcmp(word, "dbinfo")){
-            (*reply) = g_strdup_printf("dbinfo: %s %s %d %d %d\n",
+            (*reply) = g_strdup_printf("dbinfo: %s %s %lld %lld %lld\n",
                   (server->dataset->header->type & 1)?"dna":"protein",
                   (server->dataset->header->type & (1<<1))
                       ?"softmasked":"unmasked",
-                  (gint)server->dataset->header->number_of_seqs,
-                  (gint)server->dataset->header->max_seq_len,
-                  (gint)server->dataset->header->total_seq_len);
+                  server->dataset->header->number_of_seqs,
+                  server->dataset->header->max_seq_len,
+                  server->dataset->header->total_seq_len);
         } else if(!strcmp(word, "lookup")){
             if(word_list->len == 2){
                 id = word_list->pdata[1];
@@ -639,6 +734,13 @@ static gboolean Exonerate_Server_process(gchar *msg, gchar **reply,
             }
         }
     g_ptr_array_free(word_list, TRUE);
+    if((server->verbosity >= 3) && (*reply)){
+        if((server->verbosity == 3) && (strlen(*reply) >= 80))
+            g_print("Message: server returned reply [%.*s<truncated>]\n",
+                    80, (*reply));
+        else
+            g_print("Message: server returned reply [%s]\n", (*reply));
+        }
     return keep_connection;
     }
 
