@@ -528,6 +528,7 @@ static SparseCache_Page *Scheduler_Cache_fill_func(gint start,
                                     spair->scheduler->model->span_list->len,
                                     sizeof(Scheduler_SpanSeed*));
     page->get_func = Scheduler_Cache_get_func;
+    page->copy_func = NULL;
     return page;
     }
 
@@ -586,11 +587,12 @@ void Scheduler_SpanData_get_curr(Scheduler_SpanData *span_data,
     if(stored_seed){
        g_assert(stored_seed->query_entry == query_pos);
        if((stored_seed->target_entry + span_data->span->max_target)
-        > target_pos){ /* Still valid */
+        >= target_pos){ /* Still valid */
             if(span_data->curr_span_seed){
                 if(span_data->curr_span_seed->score
-                  < stored_seed->score)
+                  < stored_seed->score){
                     span_data->curr_span_seed = stored_seed;
+                    }
             } else {
                 span_data->curr_span_seed = stored_seed;
                 }
@@ -870,6 +872,11 @@ void Scheduler_Cell_process(Scheduler_Cell *cell,
                   rel_dst_query_pos, rel_dst_target_pos,
                   input_pos, output_pos;
     Scheduler_SpanSeed span_seed;
+    /*
+    g_print("%s %s %d %d\n", __FUNCTION__,
+            spair->scheduler->is_forward?"fwd":"rev",
+            cell->query_pos, row->target_pos);
+    */
     if(spair->scheduler->is_forward){
         src_query_pos = cell->query_pos;
         src_target_pos = row->target_pos;
@@ -970,11 +977,10 @@ void Scheduler_Cell_process(Scheduler_Cell *cell,
                         /* Thaw shadows */
                         for(j = 0;
                             j < model->total_shadow_designations;
-                            j++){
+                            j++)
                             cell->score[input_pos]
                                   [spair->scheduler->shadow_start+j]
                             = span_data->curr_span_seed->shadow_data[j];
-                            }
                         }
                     }
                 }
