@@ -3,11 +3,11 @@
 *  C4 dynamic programming library - Seeded Dynamic Programming   *
 *                                                                *
 *  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
-*  Copyright (C) 2000-2006.  All Rights Reserved.                *
+*  Copyright (C) 2000-2008.  All Rights Reserved.                *
 *                                                                *
 *  This source code is distributed under the terms of the        *
-*  GNU Lesser General Public License. See the file COPYING       *
-*  or http://www.fsf.org/copyleft/lesser.html for details        *
+*  GNU General Public License, version 3. See the file COPYING   *
+*  or http://www.gnu.org/licenses/gpl.txt for details            *
 *                                                                *
 *  If you use this code, please keep this notice intact.         *
 *                                                                *
@@ -26,7 +26,7 @@ SDP_ArgumentSet *SDP_ArgumentSet_create(Argument *arg){
     if(arg){
         as = ArgumentSet_create("Seeded Dynamic Programming options");
         ArgumentSet_add_option(as, 'x', "extensionthreshold", NULL,
-                "Gapped extension threshold", "40",
+                "Gapped extension threshold", "50",
                 Argument_parse_int, &sas.dropoff);
         ArgumentSet_add_option(as, 0, "singlepass", NULL,
                 "Generate suboptimal alignment in a single pass", "TRUE",
@@ -493,7 +493,8 @@ static GPtrArray *SDP_Pair_create_seed_list(Comparison *comparison){
     return seed_list;
     }
 
-SDP_Pair *SDP_Pair_create(SDP *sdp, Comparison *comparison,
+SDP_Pair *SDP_Pair_create(SDP *sdp, SubOpt *subopt,
+                          Comparison *comparison,
                           gpointer user_data){
     register SDP_Pair *sdp_pair = g_new(SDP_Pair, 1);
     g_assert(sdp);
@@ -503,8 +504,7 @@ SDP_Pair *SDP_Pair_create(SDP *sdp, Comparison *comparison,
     sdp_pair->comparison = Comparison_share(comparison);
     sdp_pair->user_data = user_data;
     sdp_pair->alignment_count = 0;
-    sdp_pair->subopt = SubOpt_create(comparison->query->len,
-                                     comparison->target->len);
+    sdp_pair->subopt = SubOpt_share(subopt);
     sdp_pair->seed_list = SDP_Pair_create_seed_list(comparison);
     sdp_pair->seed_list_by_score = NULL;
     sdp_pair->boundary = NULL;
@@ -804,12 +804,12 @@ Alignment *SDP_Pair_next_path(SDP_Pair *sdp_pair, C4_Score threshold){
         }
     g_assert(best_seed);
     g_assert(alignment);
+    /* Check score is less than previous */
     g_assert((sdp_pair->last_score < 0)
           || (best_seed->max_end->score <= sdp_pair->last_score));
     sdp_pair->alignment_count++;
     sdp_pair->last_score = best_seed->max_end->score;
     best_seed->max_end->score = C4_IMPOSSIBLY_LOW_SCORE;
-    SubOpt_add_alignment(sdp_pair->subopt, alignment);
     return alignment;
     }
 
