@@ -238,7 +238,6 @@ static SplicePredictor_GTAGonly *SplicePredictor_GTAGonly_create(
     return spgo;
     }
 
-
 SplicePredictor *SplicePredictor_create(SpliceType type){
     register SplicePredictor *sp = g_new(SplicePredictor, 1);
     register gint i, j, a, z;
@@ -283,7 +282,7 @@ SplicePredictor *SplicePredictor_create(SpliceType type){
          for(j = 0; j < 4; j++){
              sp->model_data[i][j] = (((gfloat)(1+sp->model_data[i][j]))
                                   /((25.0+1.0)));
-             sp->model_data[i][j] = log(sp->model_data[i][j]);
+             sp->model_data[i][j] = log(sp->model_data[i][j]) * 2.0;
              }
          sp->model_data[i][4] = 0.0;
          }
@@ -376,18 +375,22 @@ void SplicePredictor_predict_array(SplicePredictor *sp,
     return;
     }
 
+static gint SplicePredictor_round(gfloat num){
+    return (gint)((num < 0)?(num - 0.5):(num + 0.5));
+    }
+
 void SplicePredictor_predict_array_int(SplicePredictor *sp,
                                        gchar *seq, guint seq_len,
                                        guint start, guint length,
                                        gint *pred){
     register guint i;
+    register gfloat score;
     g_assert(seq);
     g_assert(length?(pred?TRUE:FALSE):TRUE);
     g_assert((start+length) <= seq_len);
     for(i = 0; i < length; i++){
-        pred[i] = (gint)
-                  Splice_predict_position(sp, seq, seq_len, i+start);
-        /* g_message("done PRED [%d]", pred[i]); */
+        score = Splice_predict_position(sp, seq, seq_len, i+start);
+        pred[i] = SplicePredictor_round(score);
         }
     return;
     }
@@ -442,6 +445,7 @@ static SparseCache_Page *SplicePrediction_cache_fill_func(gint start,
     register gint seq_start, seq_length, pred_start, pred_length;
     register gchar *seq;
     page->get_func = SplicePredictor_cache_get_func;
+    page->copy_func = NULL;
     pred_start = MAX(0, start);
     pred_length = MIN(spn->length, start+SparseCache_PAGE_SIZE)
                 - pred_start;
