@@ -138,7 +138,7 @@ static gchar *Translate_convert_genetic_code(gchar *code){
                 result[n++] = code[(table[a] << 4)
                                  | (table[b] << 2)
                                  |  table[c]];
-    g_print("generated genetic code [%s]\n", result);
+    /* g_print("generated genetic code [%s]\n", result); */
     return result;
     }
 /* Converts genetic code from NCBI format (TCAG order)
@@ -242,9 +242,16 @@ Translate *Translate_share(Translate *t){
     }
 
 void Translate_destroy(Translate *t){
+    register gint i;
     g_assert(t);
     if(--t->ref_count)
         return;
+    for(i = 0; t->code[i]; i++){
+        if(t->revtrans[t->code[i]]){
+            g_ptr_array_free(t->revtrans[t->code[i]], TRUE);
+            t->revtrans[t->code[i]] = NULL;
+            }
+        }
     g_free(t->code);
     g_free(t);
     return;
