@@ -40,11 +40,9 @@ FSM *FSM_create(gchar *alphabet, FSM_Join_Func merge_func,
     f->merge_func = merge_func;
     f->combine_func = combine_func;
     f->user_data = user_data;
-    f->memchunk = g_mem_chunk_new("fsm",
-                   sizeof(FSM_Node)*f->width,
-                   sizeof(FSM_Node)*f->width*1024, G_ALLOC_ONLY);
+    f->recycle = RecycleBin_create("FSM", sizeof(FSM_Node)*f->width, 1024);
     f->chunk_count = 1;
-    f->root = g_mem_chunk_alloc0(f->memchunk);
+    f->root = RecycleBin_alloc_blank(f->recycle);
     f->is_compiled = FALSE;
     return f;
     }
@@ -61,8 +59,8 @@ void FSM_info(FSM *f){
             " Alphabet size   = %d\n"
             " Size of node    = %d\n"
             " Size of chunk   = %d\n"
-            " Chunks used     = %ld\n"
-            " Bytes used      = %ld\n",
+            " Chunks used     = %lu\n"
+            " Bytes used      = %lu\n",
             f->width,
             (gint)sizeof(FSM_Node),
             (gint)sizeof(FSM_Node)*f->width,
@@ -72,7 +70,7 @@ void FSM_info(FSM *f){
     }
 
 void FSM_destroy(FSM *f){
-    g_mem_chunk_destroy(f->memchunk);
+    RecycleBin_destroy(f->recycle);
     g_free(f);
     return;
     }
@@ -112,7 +110,7 @@ gpointer FSM_add(FSM *f, gchar *seq, guint len, gpointer node_data){
             g_error("Illegal char (%d)[%c] in FSM word [%s]",
                     *useq, *useq, seq);
         if(!*(ptp = &n[f->insertion_filter[*useq]].next)){
-            *ptp = g_mem_chunk_alloc0(f->memchunk);
+            *ptp = RecycleBin_alloc_blank(f->recycle);
             f->chunk_count++;
             }
         n = *ptp;
