@@ -22,6 +22,7 @@ extern "C" {
 
 #include <limits.h>
 #include <glib.h> /* For CHAR_BIT */
+#include "recyclebin.h"
 
 #ifndef ALPHABETSIZE
 #define ALPHABETSIZE (1<<(CHAR_BIT))
@@ -43,7 +44,7 @@ typedef struct {
        FSM_Node *root;         /* Base of the finite state machine   */
   FSM_Join_Func  merge_func;   /* Function for node data merging     */
   FSM_Join_Func  combine_func; /* Function for node data combination */
-      GMemChunk *memchunk;     /* Memchunk for node chunks           */
+     RecycleBin *recycle;      /* RecycleBin for node chunks         */
          gulong  chunk_count;  /* Number of chunks used              */
        gpointer  user_data;
        gboolean  is_compiled;
