@@ -30,23 +30,21 @@ RangeTree *RangeTree_create(void){
     register RangeTree *rt = g_new(RangeTree, 1);
     rt->root = NULL;
     rt->recent_data = g_tree_new(RangeTree_recent_data_compare);
-    rt->node_memchunk = g_mem_chunk_new("RangeTree",
-            sizeof(RangeTree_Node),
-            sizeof(RangeTree_Node)*256, G_ALLOC_ONLY);
+    rt->node_recycle = RecycleBin_create("RangeTree_Node",
+                                         sizeof(RangeTree_Node), 256);
     return rt;
     }
 
 void RangeTree_destroy(RangeTree *rt, RangeTree_FreeFunc rtff,
                               gpointer user_data){
     g_tree_destroy(rt->recent_data);
-    g_mem_chunk_destroy(rt->node_memchunk);
+    RecycleBin_destroy(rt->node_recycle);
     g_free(rt);
     return;
     }
 
 void RangeTree_add(RangeTree *rt, gint x, gint y, gpointer info){
-    register RangeTree_Node *rtn = g_chunk_new(RangeTree_Node,
-                                               rt->node_memchunk);
+    register RangeTree_Node *rtn = RecycleBin_alloc(rt->node_recycle);
     g_assert(!RangeTree_check_pos(rt, x, y));
     rtn->x = x;
     rtn->y = y;
@@ -202,3 +200,25 @@ gboolean RangeTree_check_pos(RangeTree *rt,
                                    RangeTree_find_point, NULL);
     }
 
+gboolean RangeTree_is_empty(RangeTree *rt){
+    RangeTree_insert_recent(rt);
+    return rt->root?FALSE:TRUE;
+    }
+
+static gboolean RangeTree_traverse_recur(RangeTree_Node *rtn,
+                                     RangeTree_ReportFunc rtrf,
+                                     gpointer user_data){
+    if(!rtn)
+        return FALSE;
+    if(RangeTree_traverse_recur(rtn->left, rtrf, user_data)
+    || rtrf(rtn->x, rtn->y, rtn->info, user_data)
+    || RangeTree_traverse_recur(rtn->right, rtrf, user_data))
+        return TRUE;
+    return FALSE;
+    }
+
+gboolean RangeTree_traverse(RangeTree *rt,
+                        RangeTree_ReportFunc rtrf, gpointer user_data){
+    return RangeTree_traverse_recur(rt->root, rtrf, user_data);
+    }
+
