@@ -21,6 +21,7 @@ extern "C" {
 #endif /* __cplusplus */
 
 #include <glib.h>
+#include "recyclebin.h"
 
 typedef struct RangeTree_Node {
     struct RangeTree_Node *left;
@@ -33,7 +34,7 @@ typedef struct RangeTree_Node {
 typedef struct {
     RangeTree_Node *root;
              GTree *recent_data;
-         GMemChunk *node_memchunk;
+        RecycleBin *node_recycle;
 } RangeTree;
 
 RangeTree *RangeTree_create(void);
@@ -59,6 +60,11 @@ gboolean RangeTree_find(RangeTree *rt, gint x_start, gint x_length,
 gboolean RangeTree_check_pos(RangeTree *rt, gint x, gint y);
 /* Check a single position without refilling RangeTree */
 
+gboolean RangeTree_is_empty(RangeTree *rt);
+
+gboolean RangeTree_traverse(RangeTree *rt,
+                            RangeTree_ReportFunc rtrf, gpointer user_data);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
