@@ -13,6 +13,7 @@
 *                                                                *
 \****************************************************************/
 
+#include <string.h>
 #include "recyclebin.h"
 
 static GTree *global_recycle_bin_tree = NULL;
@@ -99,6 +100,12 @@ gpointer RecycleBin_alloc(RecycleBin *recycle_bin){
     return (gpointer)node;
     }
 
+gpointer RecycleBin_alloc_blank(RecycleBin *recycle_bin){
+    register gpointer data = RecycleBin_alloc(recycle_bin);
+    memset(data, 0, recycle_bin->node_size);
+    return data;
+    }
+
 void RecycleBin_recycle(RecycleBin *recycle_bin, gpointer data){
     register RecycleBin_Node *node = data;
     g_assert(node != recycle_bin->recycle);
@@ -115,7 +122,7 @@ static gint RecycleBin_profile_traverse(gpointer key,
     g_assert(recycle_bin);
     g_message("  RecycleBin [%s] %d Mb (%d items)",
               recycle_bin->name,
-              RecycleBin_memory_usage(recycle_bin)>>20,
+              (gint)(RecycleBin_memory_usage(recycle_bin)>>20),
               recycle_bin->count);
     return FALSE;
     }
