@@ -71,18 +71,24 @@ SparseCache *SparseCache_share(SparseCache *sc){
     return sc;
     }
 
+static SparseCache_Page *SparseCache_fill(SparseCache *sc, gint page_id){
+    register SparseCache_Page *page;
+    g_assert(!sc->page_list[page_id]);
+    page = sc->fill_func((page_id << SparseCache_PAGE_SIZE_BIT_WIDTH),
+                         sc->user_data);
+    sc->page_list[page_id] = page;
+    sc->page_used++;
+    sc->page_memory_usage += (sizeof(SparseCache_Page)+page->data_size);
+    return page;
+    }
+
 gpointer SparseCache_get(SparseCache *sc, gint pos){
     register gint page_id = SparseCache_pos2page(pos);
     register SparseCache_Page *page = sc->page_list[page_id];
     g_assert(pos >= 0);
     g_assert(pos < sc->length);
-    if(!page){
-        page = sc->fill_func((page_id << SparseCache_PAGE_SIZE_BIT_WIDTH),
-                             sc->user_data);
-        sc->page_list[page_id] = page;
-        sc->page_used++;
-        sc->page_memory_usage += (sizeof(SparseCache_Page)+page->data_size);
-        }
+    if(!page)
+        page = SparseCache_fill(sc, page_id);
     return page->get_func((pos & (SparseCache_PAGE_SIZE-1)),
                           page->data, sc->user_data);
     }
@@ -93,5 +99,27 @@ gsize SparseCache_memory_usage(SparseCache *sc){
          + sc->page_memory_usage;
     }
 
+void SparseCache_copy(SparseCache *sc, gint start, gint length,
+                      gchar *dst){
+    register gint pos = start, dst_pos = 0, page_id, copy_len,
+                  read_remain = length, page_remain, page_start;
+    register SparseCache_Page *page;
+    do {
+        page_id = SparseCache_pos2page(pos);
+        page = sc->page_list[page_id];
+        if(!page)
+            page = SparseCache_fill(sc, page_id);
+        page_start = page_id*SparseCache_PAGE_SIZE;
+        page_remain = page_start+SparseCache_PAGE_SIZE-pos;
+        copy_len = MIN(page_remain, read_remain);
+        g_assert(page->copy_func);
+        dst_pos += page->copy_func(pos-page_start, copy_len, dst+dst_pos,
+                                   page->data, sc->user_data);
+        read_remain -= copy_len;
+        pos += copy_len;
+    } while(read_remain > 0);
+    return;
+    }
+
 /**/
 
