@@ -29,11 +29,14 @@ extern "C" {
 
 typedef gpointer (*SparseCache_GetFunc)(gint pos, gpointer page_data,
                                         gpointer user_data);
+typedef gint (*SparseCache_CopyFunc)(gint start, gint length, gchar *dst,
+                                     gpointer page_data, gpointer user_data);
 
 typedef struct {
-                  gpointer data;
-       SparseCache_GetFunc get_func;
-                     gsize data_size;
+                   gpointer data;
+        SparseCache_GetFunc get_func;
+       SparseCache_CopyFunc copy_func;
+                      gsize data_size;
 } SparseCache_Page;
 
 typedef SparseCache_Page *(*SparseCache_FillFunc)(gint start,
@@ -68,6 +71,8 @@ SparseCache *SparseCache_create(gint length,
 SparseCache *SparseCache_share(SparseCache *sc);
     gpointer SparseCache_get(SparseCache *sc, gint pos);
        gsize SparseCache_memory_usage(SparseCache *sc);
+        void SparseCache_copy(SparseCache *sc, gint start, gint length,
+                              gchar *dst);
 
 #ifdef __cplusplus
 }
