@@ -44,7 +44,7 @@ static gboolean is_stop_codon(gchar *seq){
 static gboolean fasta_valid_cds_traverse_func(FastaDB_Seq *fdbs,
                                               gpointer user_data){
     register gint i;
-    register gboolean explain = user_data?TRUE:FALSE;
+    register gboolean explain = *((gboolean*)user_data);
     register gchar *seq;
     if(fdbs->seq->len % 3){
         if(explain)
@@ -125,8 +125,10 @@ int Argument_main(Argument *arg){
         "Guy St.C. Slater. guy@ebi.ac.uk. 2004.\n", NULL);
     fdb = FastaDB_open(query_path, NULL);
     FastaDB_traverse(fdb, FastaDB_Mask_ALL,
-                     fasta_valid_cds_traverse_func, (gpointer)explain);
+                         fasta_valid_cds_traverse_func, &explain);
     FastaDB_close(fdb);
     return 0;
     }
 
+/**/
+
